#include "deviceinfownd.h"

#include <ctype.h>

// -------------------------------------------------------------------------
// Start of IDeviceInfoWnd interface
// -------------------------------------------------------------------------

void
CDeviceInfoWnd::deviceChanged() {
#if !defined(HWSIM)
  ADMXRC3_BANK_INFO bankInfo;
#endif
  ADMXRC3_CARD_INFOEX cardInfoEx;
#if !defined(HWSIM)
  ADMXRC3_FLASH_INFO flashInfo;
#endif
  ADMXRC3_FPGA_INFO fpgaInfo;
#if !defined(HWSIM)
  ADMXRC3_MODULE_INFO moduleInfo;
#endif
  ADMXRC3_SENSOR_INFO sensorInfo;
  ADMXRC3_VERSION_INFO versionInfo;
  ADMXRC3_WINDOW_INFO windowInfo;
  ADMXRC3_STATUS status;
  std::ostringstream ss;

  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  if (m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE) {
    for (unsigned int i = 0; i < m_numTargetFpga; i++) {
      m_model->erase(m_rows.m_targetFpgaChildren[i].m_fpga);
    }
    for (unsigned int i = 0; i < m_numWindow; i++) {
      m_model->erase(m_rows.m_windowChildren[i].m_window);
    }
    for (unsigned int i = 0; i < m_numFlashBank; i++) {
      m_model->erase(m_rows.m_flashChildren[i].m_flash);
    }
    for (unsigned int i = 0; i < m_numMemoryBank; i++) {
      m_model->erase(m_rows.m_memoryChildren[i].m_memory);
    }
    for (unsigned int i = 0; i < m_numSensor; i++) {
      m_model->erase(m_rows.m_sensorChildren[i].m_sensor);
    }
    for (unsigned int i = 0; i < m_numModuleSite; i++) {
      m_model->erase(m_rows.m_moduleChildren[i].m_module);
    }
  }
  m_numTargetFpga = 0;
  m_numWindow = 0;
  m_numFlashBank = 0;
  m_numMemoryBank = 0;
  m_numModuleSite = 0;
  m_numSensor = 0;

  m_hDevice = devInfo.m_hDevice;

  // API version information subtree
  if (ADMXRC3_HANDLE_INVALID_VALUE != m_hDevice) {
    status = ADMXRC3_GetVersionInfo(devInfo.m_hDevice, &versionInfo);
    if (ADMXRC3_SUCCESS == status) {
      ss.str("");
      ss << versionInfo.Api.Major << "." << versionInfo.Api.Minor << "." << versionInfo.Api.Bugfix;
      setValue(m_rows.m_apiChildren.m_api, ss);
      ss.str("");
      ss << versionInfo.Driver.Major << "." << versionInfo.Driver.Minor << "." << versionInfo.Driver.Bugfix;
      setValue(m_rows.m_apiChildren.m_driver, ss);
    } else {
      ss.str("");
      ss << "(Unable to determine: " << ADMXRC3_GetStatusString(status, TRUE) << ")";
      setValue(m_rows.m_apiChildren.m_api, ss);
      ss.str("");
      ss << "(Unable to determine: " << ADMXRC3_GetStatusString(status, TRUE) << ")";
      setValue(m_rows.m_apiChildren.m_driver, ss);
    }
  } else {
    ss.str("(Could not open device)");
    setValue(m_rows.m_apiChildren.m_api, ss);
    setValue(m_rows.m_apiChildren.m_driver, ss);
  }

  // Summary information subtree
  if (ADMXRC3_HANDLE_INVALID_VALUE != m_hDevice) {
    status = ADMXRC3_GetCardInfoEx(devInfo.m_hDevice, &cardInfoEx);
    if (ADMXRC3_SUCCESS == status) {
      const TCHAR* pTmp;

      pTmp = CAdmxrc3Help::getModelName(cardInfoEx.Model);
      ss.str("");
      ss << cardInfoEx.Model << " (0x" << HEX << cardInfoEx.Model << DEC << ") => " << ((pTmp != NULL) ? pTmp : "unknown");
      setValue(m_rows.m_summaryChildren[0], ss);
      ss.str("");
      ss << cardInfoEx.SerialNumber << " (0x" << HEX << cardInfoEx.SerialNumber << DEC << ")";
      setValue(m_rows.m_summaryChildren[1], ss);
      ss.str("");
      ss << cardInfoEx.NumTargetFpga;
      setValue(m_rows.m_summaryChildren[2], ss);
      ss.str("");
      ss << cardInfoEx.NumClockGen;
      setValue(m_rows.m_summaryChildren[3], ss);
      ss.str("");
      ss << cardInfoEx.NumDmaChannel;
      setValue(m_rows.m_summaryChildren[4], ss);
      ss.str("");
      ss << cardInfoEx.NumWindow;
      setValue(m_rows.m_summaryChildren[5], ss);
      ss.str("");
      ss << cardInfoEx.NumSensor;
      setValue(m_rows.m_summaryChildren[6], ss);
      ss.str("");
      ss << cardInfoEx.NumModuleSite;
      setValue(m_rows.m_summaryChildren[7], ss);
      ss.str("");
      ss << cardInfoEx.NumFlashBank;
      setValue(m_rows.m_summaryChildren[8], ss);
      ss.str("");
      ss << cardInfoEx.NumMemoryBank;
      setValue(m_rows.m_summaryChildren[9], ss);
      ss.str("");
      ss << cardInfoEx.MemoryBankPresent;
      setValue(m_rows.m_summaryChildren[10], ss);

      // Target FPGA subtree
      m_numTargetFpga = cardInfoEx.NumTargetFpga;
      for (unsigned int i = 0; i < m_numTargetFpga && i < m_maxNumTargetFpga; i++) {
        status = ADMXRC3_GetFpgaInfo(devInfo.m_hDevice, i, &fpgaInfo);
        m_rows.m_targetFpgaChildren[i].m_fpga = *m_model->append(m_rows.m_targetFpga.children());
        if (status == ADMXRC3_SUCCESS) {
          if (fpgaInfo.Present) {
            const TCHAR* pTmp;
            std::string identifier(fpgaInfo.Identifier);
            std::ostringstream ss;

            std::transform(identifier.begin(), identifier.end(), identifier.begin(), ::toupper);
            ss.str("Target FPGA");
            ss << i << "  -  " << identifier;
            if (fpgaInfo.Flags & ADMXRC3_FPGA_SPEEDVALID) {
              ss << "-" << fpgaInfo.SpeedGrade;
            }
            if (fpgaInfo.Flags & ADMXRC3_FPGA_STEPPINGVALID) {
              ss << "-" << fpgaInfo.Stepping;
            }
            setProperty(m_rows.m_targetFpgaChildren[i].m_fpga, ss);

            m_rows.m_targetFpgaChildren[i].m_identifier = *m_model->append(m_rows.m_targetFpgaChildren[i].m_fpga.children());
            ss.str("Identifier: ");
            setProperty(m_rows.m_targetFpgaChildren[i].m_identifier, ss);
            ss.str(fpgaInfo.Identifier);
            setValue(m_rows.m_targetFpgaChildren[i].m_identifier, ss);

            m_rows.m_targetFpgaChildren[i].m_deviceCode = *m_model->append(m_rows.m_targetFpgaChildren[i].m_fpga.children());
            ss.str("Device code: ");
            setProperty(m_rows.m_targetFpgaChildren[i].m_deviceCode, ss);
            ss.str("");
            ss << fpgaInfo.DeviceCode << " (0x" << HEX << fpgaInfo.DeviceCode << ")" << DEC;
            setValue(m_rows.m_targetFpgaChildren[i].m_deviceCode, ss);

            pTmp = CAdmxrc3Help::getFamilyString(fpgaInfo.FamilyCode);
            m_rows.m_targetFpgaChildren[i].m_familyCode = *m_model->append(m_rows.m_targetFpgaChildren[i].m_fpga.children());
            ss.str("Family code: ");
            setProperty(m_rows.m_targetFpgaChildren[i].m_familyCode, ss);
            ss.str("");
            ss << fpgaInfo.FamilyCode << " (0x" << HEX << fpgaInfo.FamilyCode << ")" << DEC << " => " << ((NULL != pTmp) ? pTmp : "unknown");
            setValue(m_rows.m_targetFpgaChildren[i].m_familyCode, ss);

            pTmp = CAdmxrc3Help::getSubfamilyString(fpgaInfo.SubfamilyCode);
            m_rows.m_targetFpgaChildren[i].m_subfamilyCode = *m_model->append(m_rows.m_targetFpgaChildren[i].m_fpga.children());
            ss.str("Subfamily code: ");
            setProperty(m_rows.m_targetFpgaChildren[i].m_subfamilyCode, ss);
            ss.str("");
            ss << fpgaInfo.SubfamilyCode << " (0x" << HEX << fpgaInfo.SubfamilyCode << ")" << DEC << " => " << ((NULL != pTmp) ? pTmp : "unknown");
            setValue(m_rows.m_targetFpgaChildren[i].m_subfamilyCode, ss);

            m_rows.m_targetFpgaChildren[i].m_packageCode = *m_model->append(m_rows.m_targetFpgaChildren[i].m_fpga.children());
            ss.str("Package code: ");
            setProperty(m_rows.m_targetFpgaChildren[i].m_packageCode, ss);
            ss.str("");
            ss << fpgaInfo.PackageCode << "(0x " << HEX << fpgaInfo.PackageCode << DEC << ")" << " => " <<
              ADMXRC3_PACKAGE_GETTYPE0(fpgaInfo.PackageCode) << ADMXRC3_PACKAGE_GETTYPE1(fpgaInfo.PackageCode) << ADMXRC3_PACKAGE_GETPINS(fpgaInfo.PackageCode);
            setValue(m_rows.m_targetFpgaChildren[i].m_packageCode, ss);

            m_rows.m_targetFpgaChildren[i].m_device = *m_model->append(m_rows.m_targetFpgaChildren[i].m_fpga.children());
            ss.str("Device: ");
            setProperty(m_rows.m_targetFpgaChildren[i].m_device, ss);
            ss.str(fpgaInfo.Device);
            setValue(m_rows.m_targetFpgaChildren[i].m_device, ss);

            m_rows.m_targetFpgaChildren[i].m_package = *m_model->append(m_rows.m_targetFpgaChildren[i].m_fpga.children());
            ss.str("Package: ");
            setProperty(m_rows.m_targetFpgaChildren[i].m_package, ss);
            ss.str(fpgaInfo.Package);
            setValue(m_rows.m_targetFpgaChildren[i].m_package, ss);

            m_rows.m_targetFpgaChildren[i].m_flags = *m_model->append(m_rows.m_targetFpgaChildren[i].m_fpga.children());
            ss.str("Flags: ");
            setProperty(m_rows.m_targetFpgaChildren[i].m_flags, ss);
            ss.str("");
            ss << "0x" << HEX << fpgaInfo.Flags << DEC;
            setValue(m_rows.m_targetFpgaChildren[i].m_flags, ss);
            if (fpgaInfo.Flags & ADMXRC3_FPGA_SPEEDVALID) {
              m_rows.m_targetFpgaChildren[i].m_flagsChildren.m_speed = *m_model->append(m_rows.m_targetFpgaChildren[i].m_flags.children());
              ss.str("Speed grade valid");
              setProperty(m_rows.m_targetFpgaChildren[i].m_flagsChildren.m_speed, ss);
              ss.str("");
              ss << "0x" << HEX << ADMXRC3_FPGA_SPEEDVALID << DEC;
              setValue(m_rows.m_targetFpgaChildren[i].m_flagsChildren.m_speed, ss);
            }
            if (fpgaInfo.Flags & ADMXRC3_FPGA_STEPPINGVALID) {
              m_rows.m_targetFpgaChildren[i].m_flagsChildren.m_stepping = *m_model->append(m_rows.m_targetFpgaChildren[i].m_flags.children());
              ss.str("Stepping valid");
              setProperty(m_rows.m_targetFpgaChildren[i].m_flagsChildren.m_stepping, ss);
              ss.str("");
              ss << "0x" << HEX << ADMXRC3_FPGA_STEPPINGVALID << DEC;
              setValue(m_rows.m_targetFpgaChildren[i].m_flagsChildren.m_stepping, ss);
            }
            if (fpgaInfo.Flags & ADMXRC3_FPGA_NOTCONFIGURABLE) {
              m_rows.m_targetFpgaChildren[i].m_flagsChildren.m_notConfigurable = *m_model->append(m_rows.m_targetFpgaChildren[i].m_flags.children());
              ss.str("Not configurable");
              setProperty(m_rows.m_targetFpgaChildren[i].m_flagsChildren.m_notConfigurable, ss);
            }
          } else {
            ss.str("Target FPGA");
            ss << i << "  -  (Not present)";
            setProperty(m_rows.m_targetFpgaChildren[i].m_fpga, ss);
          }
        } else {
          ss.str("Target FPGA");
          ss << i << "  -  (Unable to determine: " << ADMXRC3_GetStatusString(status, TRUE) << ")";
          setProperty(m_rows.m_targetFpgaChildren[i].m_fpga, ss);
        }
      }

      // Window subtree
      m_numWindow = cardInfoEx.NumWindow;
      for (unsigned int i = 0; i < m_numWindow && i < m_maxNumWindow; i++) {
        const TCHAR* pTmp;

        m_rows.m_windowChildren[i].m_window = *m_model->append(m_rows.m_window.children());
        status = ADMXRC3_GetWindowInfo(devInfo.m_hDevice, i, &windowInfo);
        if (status == ADMXRC3_SUCCESS) {
          pTmp = CAdmxrc3Help::getWindowDescription(i, cardInfoEx.Model);
          ss.str("");
          ss << i << "  -  " << ((NULL != pTmp) ? pTmp : "Unknown");
          setProperty(m_rows.m_windowChildren[i].m_window, ss);

          m_rows.m_windowChildren[i].m_busBase = *m_model->append(m_rows.m_windowChildren[i].m_window.children());
          ss.str("Bus base address");
          setProperty(m_rows.m_windowChildren[i].m_busBase, ss);
          ss.str("");
          ss << "0x" << HEX << windowInfo.BusBase << DEC;
          setValue(m_rows.m_windowChildren[i].m_busBase, ss);

          m_rows.m_windowChildren[i].m_busLimit = *m_model->append(m_rows.m_windowChildren[i].m_window.children());
          ss.str("Bus limit address");
          setProperty(m_rows.m_windowChildren[i].m_busLimit, ss);
          ss.str("");
          ss << "0x" << HEX << (windowInfo.BusBase + windowInfo.BusSize - 1) << DEC;
          setValue(m_rows.m_windowChildren[i].m_busLimit, ss);

          m_rows.m_windowChildren[i].m_busSize = *m_model->append(m_rows.m_windowChildren[i].m_window.children());
          ss.str("Bus size");
          setProperty(m_rows.m_windowChildren[i].m_busSize, ss);
          ss.str("");
          ss << "0x" << HEX << windowInfo.BusSize << DEC;
          setValue(m_rows.m_windowChildren[i].m_busSize, ss);

          if (windowInfo.LocalSize) {
            m_rows.m_windowChildren[i].m_localBase = *m_model->append(m_rows.m_windowChildren[i].m_window.children());
            ss.str("Local base address");
            setProperty(m_rows.m_windowChildren[i].m_localBase, ss);
            ss.str("");
            ss << "0x" << HEX << windowInfo.LocalBase << DEC;
            setValue(m_rows.m_windowChildren[i].m_localBase, ss);

            m_rows.m_windowChildren[i].m_localLimit = *m_model->append(m_rows.m_windowChildren[i].m_window.children());
            ss.str("Local limit address");
            setProperty(m_rows.m_windowChildren[i].m_localLimit, ss);
            ss.str("");
            ss << "0x" << HEX << (windowInfo.LocalBase + windowInfo.LocalSize - 1) << DEC;
            setValue(m_rows.m_windowChildren[i].m_localLimit, ss);

            m_rows.m_windowChildren[i].m_localSize = *m_model->append(m_rows.m_windowChildren[i].m_window.children());
            ss.str("Local size");
            setProperty(m_rows.m_windowChildren[i].m_localSize, ss);
            ss.str("");
            ss << "0x" << HEX << windowInfo.LocalSize << DEC;
            setValue(m_rows.m_windowChildren[i].m_localSize, ss);
          }

          m_rows.m_windowChildren[i].m_virtualSize = *m_model->append(m_rows.m_windowChildren[i].m_window.children());
          ss.str("Virtual size");
          setProperty(m_rows.m_windowChildren[i].m_virtualSize, ss);
          ss.str("");
          ss << "0x" << HEX << windowInfo.VirtualSize << DEC;
          setValue(m_rows.m_windowChildren[i].m_virtualSize, ss);
        } else {
          ss.str("");
          ss << i << "  -  (Unable to determine: " << ADMXRC3_GetStatusString(status, TRUE) << ")";
          setProperty(m_rows.m_windowChildren[i].m_window, ss);
        }
      }

      // Sensor subtree
      m_numSensor = cardInfoEx.NumSensor;
      for (unsigned int i = 0; i < m_numSensor && i < m_maxNumSensor; i++) {
        const TCHAR* pTmp;

        m_rows.m_sensorChildren[i].m_sensor = *m_model->append(m_rows.m_sensor.children());
        status = ADMXRC3_GetSensorInfo(devInfo.m_hDevice, i, &sensorInfo);
        if (status == ADMXRC3_SUCCESS) {
          ss.str("");
          ss << i << "  -  " << sensorInfo.Description;
          setProperty(m_rows.m_sensorChildren[i].m_sensor, ss);

          m_rows.m_sensorChildren[i].m_description = *m_model->append(m_rows.m_sensorChildren[i].m_sensor.children());
          ss.str("Description");
          setProperty(m_rows.m_sensorChildren[i].m_description, ss);
          ss.str(sensorInfo.Description);
          setValue(m_rows.m_sensorChildren[i].m_description, ss);

          pTmp = CAdmxrc3Help::getUnitString(sensorInfo.Unit);
          m_rows.m_sensorChildren[i].m_unit = *m_model->append(m_rows.m_sensorChildren[i].m_sensor.children());
          ss.str("Unit");
          setProperty(m_rows.m_sensorChildren[i].m_unit, ss);
          ss.str("");
          ss << sensorInfo.Unit << " (0x" << HEX << sensorInfo.Unit << DEC << ") => " << ((NULL != pTmp) ? pTmp : "unknown");
          setValue(m_rows.m_sensorChildren[i].m_unit, ss);

          pTmp = CAdmxrc3Help::getDatatypeString(sensorInfo.DataType);
          m_rows.m_sensorChildren[i].m_datatype = *m_model->append(m_rows.m_sensorChildren[i].m_sensor.children());
          ss.str("Datatype");
          setProperty(m_rows.m_sensorChildren[i].m_datatype, ss);
          ss.str("");
          ss << sensorInfo.DataType << " (0x" << HEX << sensorInfo.DataType << DEC << ") => " << ((NULL != pTmp) ? pTmp : "unknown");
          setValue(m_rows.m_sensorChildren[i].m_datatype, ss);

          m_rows.m_sensorChildren[i].m_exponent = *m_model->append(m_rows.m_sensorChildren[i].m_sensor.children());
          ss.str("Exponent");
          setProperty(m_rows.m_sensorChildren[i].m_exponent, ss);
          ss.str("");
          ss << sensorInfo.Exponent;
          setValue(m_rows.m_sensorChildren[i].m_exponent, ss);

          m_rows.m_sensorChildren[i].m_error = *m_model->append(m_rows.m_sensorChildren[i].m_sensor.children());
          ss.str("Error");
          setProperty(m_rows.m_sensorChildren[i].m_error, ss);
          ss.str("");
          ss << sensorInfo.Error;
          setValue(m_rows.m_sensorChildren[i].m_error, ss);

          m_rows.m_sensorChildren[i].m_capabilities = *m_model->append(m_rows.m_sensorChildren[i].m_sensor.children());
          ss.str("Capabilities");
          setProperty(m_rows.m_sensorChildren[i].m_capabilities, ss);
          ss.str("");
          ss << "0x" << HEX << sensorInfo.Capabilities << DEC;
          setValue(m_rows.m_sensorChildren[i].m_capabilities, ss);
          if (sensorInfo.Capabilities & ADMXRC3_SENSOR_SWVALUE) {
            m_rows.m_sensorChildren[i].m_capabilitiesChildren.m_swValue = *m_model->append(m_rows.m_sensorChildren[i].m_capabilities.children());
            ss.str("Software sensor");
            setProperty(m_rows.m_sensorChildren[i].m_capabilitiesChildren.m_swValue, ss);
            ss.str("");
            ss << "0x" << HEX << ADMXRC3_SENSOR_SWVALUE << DEC;
            setValue(m_rows.m_sensorChildren[i].m_capabilitiesChildren.m_swValue, ss);
          }
        } else {
          ss.str("");
          ss << i << "  -   (Unable to determine: " << ADMXRC3_GetStatusString(status, TRUE) << ")";
          setProperty(m_rows.m_sensorChildren[i].m_sensor, ss);
        }
      }

#if !defined(HWSIM)
      // Flash subtree
      m_numFlashBank = cardInfoEx.NumFlashBank;
      for (unsigned int i = 0; i < m_numFlashBank && i < m_maxNumFlashBank; i++) {

        m_rows.m_flashChildren[i].m_flash = *m_model->append(m_rows.m_flash.children());
        status = ADMXRC3_GetFlashInfo(devInfo.m_hDevice, i, &flashInfo);
        if (status == ADMXRC3_SUCCESS) {
          ss.str("");
          ss << i << "  -  " << flashInfo.Identifier;
          setProperty(m_rows.m_flashChildren[i].m_flash, ss);

          m_rows.m_flashChildren[i].m_identifier = *m_model->append(m_rows.m_flashChildren[i].m_flash.children());
          ss.str("Identifier");
          setProperty(m_rows.m_flashChildren[i].m_identifier, ss);
          ss.str(flashInfo.Identifier);
          setValue(m_rows.m_flashChildren[i].m_identifier, ss);

          m_rows.m_flashChildren[i].m_size = *m_model->append(m_rows.m_flashChildren[i].m_flash.children());
          ss.str("Size");
          setProperty(m_rows.m_flashChildren[i].m_size, ss);
          ss.str("");
          ss << (flashInfo.Size >> 10) << " (0x" << HEX << (flashInfo.Size >> 10) << DEC << ") kiByte";
          setValue(m_rows.m_flashChildren[i].m_size, ss);

          m_rows.m_flashChildren[i].m_useableStart = *m_model->append(m_rows.m_flashChildren[i].m_flash.children());
          ss.str("Useable start address");
          setProperty(m_rows.m_flashChildren[i].m_useableStart, ss);
          ss.str("");
          ss << "0x" << HEX << flashInfo.UseableStart << DEC;
          setValue(m_rows.m_flashChildren[i].m_useableStart, ss);

          m_rows.m_flashChildren[i].m_useableLimit = *m_model->append(m_rows.m_flashChildren[i].m_flash.children());
          ss.str("Useable limit address");
          setProperty(m_rows.m_flashChildren[i].m_useableLimit, ss);
          ss.str("");
          ss << "0x" << HEX << (flashInfo.UseableStart + flashInfo.UseableLength - 1ULL) << DEC;
          setValue(m_rows.m_flashChildren[i].m_useableLimit, ss);

          m_rows.m_flashChildren[i].m_useableLength = *m_model->append(m_rows.m_flashChildren[i].m_flash.children());
          ss.str("Useable length");
          setProperty(m_rows.m_flashChildren[i].m_useableLength, ss);
          ss.str("");
          ss << "0x" << HEX << flashInfo.UseableLength << DEC;
          setValue(m_rows.m_flashChildren[i].m_useableLength, ss);
        } else {
          ss.str("");
          ss << i << "  -  (Unable to determine: " << ADMXRC3_GetStatusString(status, TRUE) << ")";
          setProperty(m_rows.m_flashChildren[i].m_flash, ss);
        }
      }
      // Memory bank subtree
      m_numMemoryBank = cardInfoEx.NumMemoryBank;
      for (unsigned int i = 0; i < m_numMemoryBank && i < m_maxNumMemoryBank; i++) {
        m_rows.m_memoryChildren[i].m_memory = *m_model->append(m_rows.m_memory.children());
        status = ADMXRC3_GetBankInfo(devInfo.m_hDevice, i, &bankInfo);
        if (status == ADMXRC3_SUCCESS) {
          if (bankInfo.Present) {
            uint64_t sizeByte = bankInfo.PhysicalSize * (bankInfo.PhysicalDataWidth >> 3);

            ss.str("");
            ss << i << "  -  " << (sizeByte >> 10) << " (0x" << (sizeByte >> 10) << ") kiByte";
            setProperty(m_rows.m_memoryChildren[i].m_memory, ss);

            if (bankInfo.MaximumFrequency) {
              m_rows.m_memoryChildren[i].m_maximumFrequency = *m_model->append(m_rows.m_memoryChildren[i].m_memory.children());
              ss.str("Maximum frequency");
              setProperty(m_rows.m_memoryChildren[i].m_maximumFrequency, ss);
              ss.str("");
              ss << ((double)bankInfo.MaximumFrequency / 1.0e6f) << " MHz";
              setValue(m_rows.m_memoryChildren[i].m_maximumFrequency, ss);

              m_rows.m_memoryChildren[i].m_minimumFrequency = *m_model->append(m_rows.m_memoryChildren[i].m_memory.children());
              ss.str("Minimum frequency");
              setProperty(m_rows.m_memoryChildren[i].m_minimumFrequency, ss);
              ss.str("");
              ss << ((double)bankInfo.MinimumFrequency / 1.0e6f) << " MHz";
              setValue(m_rows.m_memoryChildren[i].m_minimumFrequency, ss);
            }

            m_rows.m_memoryChildren[i].m_physicalSize = *m_model->append(m_rows.m_memoryChildren[i].m_memory.children());
            ss.str("Physical size");
            setProperty(m_rows.m_memoryChildren[i].m_physicalSize, ss);
            ss.str("");
            ss << (bankInfo.PhysicalSize >> 10) << " (0x" << HEX << (bankInfo.PhysicalSize >> 10) << DEC << ") kiByte";
            setValue(m_rows.m_memoryChildren[i].m_physicalSize, ss);

            m_rows.m_memoryChildren[i].m_physicalDataWidth = *m_model->append(m_rows.m_memoryChildren[i].m_memory.children());
            ss.str("Physical non-ECC data width");
            setProperty(m_rows.m_memoryChildren[i].m_physicalDataWidth, ss);
            ss.str("");
            ss << bankInfo.PhysicalDataWidth << " bits";
            setValue(m_rows.m_memoryChildren[i].m_physicalDataWidth, ss);

            m_rows.m_memoryChildren[i].m_physicalEccWidth = *m_model->append(m_rows.m_memoryChildren[i].m_memory.children());
            ss.str("Physical ECC data width");
            setProperty(m_rows.m_memoryChildren[i].m_physicalEccWidth, ss);
            ss.str("");
            ss << bankInfo.PhysicalECCWidth << " bits";
            setValue(m_rows.m_memoryChildren[i].m_physicalEccWidth, ss);

            m_rows.m_memoryChildren[i].m_physicalWidth = *m_model->append(m_rows.m_memoryChildren[i].m_memory.children());
            ss.str("Physical data width");
            setProperty(m_rows.m_memoryChildren[i].m_physicalWidth, ss);
            ss.str("");
            ss << bankInfo.PhysicalWidth << " bits";
            setValue(m_rows.m_memoryChildren[i].m_physicalWidth, ss);

            m_rows.m_memoryChildren[i].m_typeMask = *m_model->append(m_rows.m_memoryChildren[i].m_memory.children());
            ss.str("Type mask");
            setProperty(m_rows.m_memoryChildren[i].m_typeMask, ss);
            ss.str("");
            ss << "0x" << HEX << bankInfo.TypeMask << DEC;
            setValue(m_rows.m_memoryChildren[i].m_typeMask, ss);
            if (bankInfo.TypeMask & ADMXRC3_BANK_ZBT_FT) {
              m_rows.m_memoryChildren[i].m_typeChildren.m_zbtFt = *m_model->append(m_rows.m_memoryChildren[i].m_typeMask.children());
              ss.str("Flowthrough ZBT");
              setProperty(m_rows.m_memoryChildren[i].m_typeChildren.m_zbtFt, ss);
              ss.str("");
              ss << "0x" << HEX << ADMXRC3_BANK_ZBT_FT << DEC;
              setValue(m_rows.m_memoryChildren[i].m_typeChildren.m_zbtFt, ss);
            }
            if (bankInfo.TypeMask & ADMXRC3_BANK_ZBT_P) {
              m_rows.m_memoryChildren[i].m_typeChildren.m_zbtP = *m_model->append(m_rows.m_memoryChildren[i].m_typeMask.children());
              ss.str("Pipelined ZBT");
              setProperty(m_rows.m_memoryChildren[i].m_typeChildren.m_zbtP, ss);
              ss.str("");
              ss << "0x" << HEX << ADMXRC3_BANK_ZBT_P << DEC;
              setValue(m_rows.m_memoryChildren[i].m_typeChildren.m_zbtP, ss);
            }
            if (bankInfo.TypeMask & ADMXRC3_BANK_SDRAM_DDR) {
              m_rows.m_memoryChildren[i].m_typeChildren.m_sdramDdr = *m_model->append(m_rows.m_memoryChildren[i].m_typeMask.children());
              ss.str("DDR SDRAM");
              setProperty(m_rows.m_memoryChildren[i].m_typeChildren.m_sdramDdr, ss);
              ss.str("");
              ss << "0x" << HEX << ADMXRC3_BANK_SDRAM_DDR << DEC;
              setValue(m_rows.m_memoryChildren[i].m_typeChildren.m_sdramDdr, ss);
            }
            if (bankInfo.TypeMask & ADMXRC3_BANK_SSRAM_DDR2) {
              m_rows.m_memoryChildren[i].m_typeChildren.m_ssramDdr2 = *m_model->append(m_rows.m_memoryChildren[i].m_typeMask.children());
              ss.str("DDR-II SSRAM");
              setProperty(m_rows.m_memoryChildren[i].m_typeChildren.m_ssramDdr2, ss);
              ss.str("");
              ss << "0x" << HEX << ADMXRC3_BANK_SSRAM_DDR2 << DEC;
              setValue(m_rows.m_memoryChildren[i].m_typeChildren.m_ssramDdr2, ss);
            }
            if (bankInfo.TypeMask & ADMXRC3_BANK_SDRAM_SDR) {
              m_rows.m_memoryChildren[i].m_typeChildren.m_sdramSdr = *m_model->append(m_rows.m_memoryChildren[i].m_typeMask.children());
              ss.str("SDR SDRAM");
              setProperty(m_rows.m_memoryChildren[i].m_typeChildren.m_sdramSdr, ss);
              ss.str("");
              ss << "0x" << HEX << ADMXRC3_BANK_SDRAM_SDR << DEC;
              setValue(m_rows.m_memoryChildren[i].m_typeChildren.m_sdramSdr, ss);
            }
            if (bankInfo.TypeMask & ADMXRC3_BANK_SDRAM_DDR2) {
              m_rows.m_memoryChildren[i].m_typeChildren.m_sdramDdr2 = *m_model->append(m_rows.m_memoryChildren[i].m_typeMask.children());
              ss.str("DDR-II SDRAM");
              setProperty(m_rows.m_memoryChildren[i].m_typeChildren.m_sdramDdr2, ss);
              ss.str("");
              ss << "0x" << HEX << ADMXRC3_BANK_SDRAM_DDR2 << DEC;
              setValue(m_rows.m_memoryChildren[i].m_typeChildren.m_sdramDdr2, ss);
            }
            if (bankInfo.TypeMask & ADMXRC3_BANK_SSRAM_QDR) {
              m_rows.m_memoryChildren[i].m_typeChildren.m_ssramQdr = *m_model->append(m_rows.m_memoryChildren[i].m_typeMask.children());
              ss.str("QDR SSRAM");
              setProperty(m_rows.m_memoryChildren[i].m_typeChildren.m_ssramQdr, ss);
              ss.str("");
              ss << "0x" << HEX << ADMXRC3_BANK_SSRAM_QDR << DEC;
              setValue(m_rows.m_memoryChildren[i].m_typeChildren.m_ssramQdr, ss);
            }
            if (bankInfo.TypeMask & ADMXRC3_BANK_SDRAM_DDR3) {
              m_rows.m_memoryChildren[i].m_typeChildren.m_sdramDdr3 = *m_model->append(m_rows.m_memoryChildren[i].m_typeMask.children());
              ss.str("DDR-3 SDRAM");
              setProperty(m_rows.m_memoryChildren[i].m_typeChildren.m_sdramDdr3, ss);
              ss.str("");
              ss << "0x" << HEX << ADMXRC3_BANK_SDRAM_DDR3 << DEC;
              setValue(m_rows.m_memoryChildren[i].m_typeChildren.m_sdramDdr3, ss);
            }

            m_rows.m_memoryChildren[i].m_connectivityMask = *m_model->append(m_rows.m_memoryChildren[i].m_memory.children());
            ss.str("Connectivity mask");
            setProperty(m_rows.m_memoryChildren[i].m_connectivityMask, ss);
            ss.str("");
            ss << "0x" << HEX << bankInfo.ConnectivityMask << DEC;
            setValue(m_rows.m_memoryChildren[i].m_connectivityMask, ss);

            for (unsigned int j = 0; j < 32; j++) {
              if (bankInfo.ConnectivityMask & (0x1U << j)) {
                m_rows.m_memoryChildren[i].m_connectivityChildren[j] = *m_model->append(m_rows.m_memoryChildren[i].m_connectivityMask.children());
                ss.str("");
                ss << "Target FPGA " << j;
                setProperty(m_rows.m_memoryChildren[i].m_connectivityChildren[j], ss);
                ss.str("");
                ss << "0x" << HEX << (0x1U << j) << DEC;
                setValue(m_rows.m_memoryChildren[i].m_connectivityChildren[j], ss);
              }
            }
          } else {
            ss.str("");
            ss << i << "  -  (Not present)";
            setProperty(m_rows.m_memoryChildren[i].m_memory, ss);
          }

          m_rows.m_memoryChildren[i].m_present = *m_model->append(m_rows.m_memoryChildren[i].m_memory.children());
          ss.str("Present");
          setProperty(m_rows.m_memoryChildren[i].m_present, ss);
          ss.str("");
          ss << bankInfo.Present;
          setValue(m_rows.m_memoryChildren[i].m_present, ss);
        } else {
          ss.str("");
          ss << i << "  -  (Unable to determine: " << ADMXRC3_GetStatusString(status, TRUE) << ")";
          setProperty(m_rows.m_memoryChildren[i].m_memory, ss);
        }
      }

      // I/O module subtree
      for (unsigned int i = 0; i < cardInfoEx.NumModuleSite; i++) {
        m_rows.m_moduleChildren[i].m_module = *m_model->append(m_rows.m_module.children());
        status = ADMXRC3_GetModuleInfo(devInfo.m_hDevice, i, &moduleInfo);
        if (status == ADMXRC3_SUCCESS) {
          if (moduleInfo.Present) {
            if (!(moduleInfo.Flags & ADMXRC3_MODULE_LEGACYXRM)) {
              /* Get manufacture date */
              struct tm tm1jan1996 = {
                0, 0, 0, 1, 0, 96, 0, 0, 0
              };
              time_t manufTime = mktime(&tm1jan1996);
              manufTime += (time_t)moduleInfo.ManufactureDate * 60;
              const struct tm* pManufDate = gmtime(&manufTime);
              char dateStr[32];
              if (NULL != pManufDate) {
                strftime(dateStr, ARRAY_LENGTH(dateStr) - 1, "%d %b %Y", pManufDate);
                dateStr[ARRAY_LENGTH(dateStr) - 1] = '\0'; /* Ensure nul-terminated */
              } else {
                dateStr[0] = '?';
                dateStr[1] = '\0';
              }

              ss.str("");
              ss << i << "  -  " << moduleInfo.Product << " #" << moduleInfo.SerialNumber;
              setProperty(m_rows.m_moduleChildren[i].m_module, ss);

              m_rows.m_moduleChildren[i].m_product = *m_model->append(m_rows.m_moduleChildren[i].m_module.children());
              ss.str("Product");
              setProperty(m_rows.m_moduleChildren[i].m_product, ss);
              ss.str("");
              ss << moduleInfo.Product;
              setValue(m_rows.m_moduleChildren[i].m_product, ss);

              m_rows.m_moduleChildren[i].m_partNumber = *m_model->append(m_rows.m_moduleChildren[i].m_module.children());
              ss.str("Part number");
              setProperty(m_rows.m_moduleChildren[i].m_partNumber, ss);
              ss.str("");
              ss << moduleInfo.PartNumber;
              setValue(m_rows.m_moduleChildren[i].m_partNumber, ss);

              m_rows.m_moduleChildren[i].m_manufacturer = *m_model->append(m_rows.m_moduleChildren[i].m_module.children());
              ss.str("Manufacturer");
              setProperty(m_rows.m_moduleChildren[i].m_manufacturer, ss);
              ss.str("");
              ss << moduleInfo.Manufacturer;
              setValue(m_rows.m_moduleChildren[i].m_manufacturer, ss);

              m_rows.m_moduleChildren[i].m_serialNumber = *m_model->append(m_rows.m_moduleChildren[i].m_module.children());
              ss.str("Serial number");
              setProperty(m_rows.m_moduleChildren[i].m_serialNumber, ss);
              ss.str("");
              ss << moduleInfo.SerialNumber;
              setValue(m_rows.m_moduleChildren[i].m_serialNumber, ss);

              m_rows.m_moduleChildren[i].m_ioVoltage = *m_model->append(m_rows.m_moduleChildren[i].m_module.children());
              ss.str("I/O voltage");
              setProperty(m_rows.m_moduleChildren[i].m_ioVoltage, ss);
              ss.str("");
              ss << moduleInfo.IoVoltage;
              setValue(m_rows.m_moduleChildren[i].m_ioVoltage, ss);

              m_rows.m_moduleChildren[i].m_manufactureDate = *m_model->append(m_rows.m_moduleChildren[i].m_module.children());
              ss.str("Manufacture date");
              setProperty(m_rows.m_moduleChildren[i].m_manufactureDate, ss);
              ss.str("");
              ss << dateStr << " (" << moduleInfo.ManufactureDate << " minutes since 00:00:00 1/1/1996)";
              setValue(m_rows.m_moduleChildren[i].m_manufactureDate, ss);
            } else {
              ss.str("");
              ss << i << "  -  Legacy XRM";;
              setProperty(m_rows.m_moduleChildren[i].m_module, ss);
            }

            m_rows.m_moduleChildren[i].m_flags = *m_model->append(m_rows.m_moduleChildren[i].m_module.children());
            ss.str("Flags");
            setProperty(m_rows.m_moduleChildren[i].m_flags, ss);
            ss.str("");
            ss << "0x" << HEX << moduleInfo.Flags << DEC;
            setValue(m_rows.m_moduleChildren[i].m_flags, ss);

            if (moduleInfo.Flags & ADMXRC3_MODULE_LEGACYXRM) {
              m_rows.m_moduleChildren[i].m_flagsItem.m_legacyXrm = *m_model->append(m_rows.m_moduleChildren[i].m_flags.children());
              ss.str("LEGACYXRM");
              setProperty(m_rows.m_moduleChildren[i].m_flagsItem.m_legacyXrm, ss);
              ss.str("");
              ss << "0x" << HEX << ADMXRC3_MODULE_LEGACYXRM << DEC;
              setValue(m_rows.m_moduleChildren[i].m_flagsItem.m_legacyXrm, ss);
            }

            if (moduleInfo.Flags & ADMXRC3_MODULE_FORCE2V5) {
              m_rows.m_moduleChildren[i].m_flagsItem.m_force2V5 = *m_model->append(m_rows.m_moduleChildren[i].m_flags.children());
              ss.str("FORCE2V5");
              setProperty(m_rows.m_moduleChildren[i].m_flagsItem.m_force2V5, ss);
              ss.str("");
              ss << "0x" << HEX << ADMXRC3_MODULE_FORCE2V5 << DEC;
              setValue(m_rows.m_moduleChildren[i].m_flagsItem.m_force2V5, ss);
            }

            if (moduleInfo.Flags & ADMXRC3_MODULE_ROMERROR) {
              m_rows.m_moduleChildren[i].m_flagsItem.m_romError = *m_model->append(m_rows.m_moduleChildren[i].m_flags.children());
              ss.str("ROMERROR");
              setProperty(m_rows.m_moduleChildren[i].m_flagsItem.m_romError, ss);
              ss.str("");
              ss << "0x" << HEX << ADMXRC3_MODULE_ROMERROR << DEC;
              setValue(m_rows.m_moduleChildren[i].m_flagsItem.m_romError, ss);
            }

            if (moduleInfo.Flags & ADMXRC3_MODULE_CSUMERROR) {
              m_rows.m_moduleChildren[i].m_flagsItem.m_cSumError = *m_model->append(m_rows.m_moduleChildren[i].m_flags.children());
              ss.str("CSUMERROR");
              setProperty(m_rows.m_moduleChildren[i].m_flagsItem.m_cSumError, ss);
              ss.str("");
              ss << "0x" << HEX << ADMXRC3_MODULE_CSUMERROR << DEC;
              setValue(m_rows.m_moduleChildren[i].m_flagsItem.m_cSumError, ss);
            }
          } else {
            ss.str("");
            ss << i << "  -  (Not present)";
            setProperty(m_rows.m_moduleChildren[i].m_module, ss);
          }
          m_rows.m_moduleChildren[i].m_present = *m_model->append(m_rows.m_moduleChildren[i].m_module.children());
          ss.str("Present");
          setProperty(m_rows.m_moduleChildren[i].m_present, ss);
          ss.str("");
          ss << (moduleInfo.Present ? "TRUE" : "FALSE");
          setValue(m_rows.m_moduleChildren[i].m_present, ss);
        } else {
          ss.str("");
          ss << i << "  -  (Unable to determine: " << ADMXRC3_GetStatusString(status, TRUE) << ")";
          setProperty(m_rows.m_moduleChildren[i].m_module, ss);
        }
      }
#endif
    } else {
      ss.str("(Unable to determine)");
      for (unsigned int i = 0; i < ARRAY_LENGTH(m_rows.m_summaryChildren); i++) {
        setValue(m_rows.m_summaryChildren[i], ss);
      }
    }
  } else {
    ss.str("(Could not open device)");
    for (unsigned int i = 0; i < ARRAY_LENGTH(m_rows.m_summaryChildren); i++) {
      setValue(m_rows.m_summaryChildren[i], ss);
    }
  }
}

void
CDeviceInfoWnd::numDeviceChanged() {
  unsigned int numDevice = m_pParent->getDeviceInfo().m_numDevice;

  if ((m_numDevice == 0) == (numDevice == 0)) {
    return;
  }

  m_model->clear();
  if (m_pParent->getDeviceInfo().m_numDevice) {
    m_rows.m_api = *m_model->append();
    m_rows.m_api[m_modelColumns.m_item] = "API information";
    m_rows.m_apiChildren.m_api = *m_model->append(m_rows.m_api.children());
    m_rows.m_apiChildren.m_api[m_modelColumns.m_item] = "API version";
    m_rows.m_apiChildren.m_driver = *m_model->append(m_rows.m_api.children());
    m_rows.m_apiChildren.m_driver[m_modelColumns.m_item] = "Driver version";
    m_rows.m_summary = *(m_model->append());
    m_rows.m_summary[m_modelColumns.m_item] = "Summary information";
    m_rows.m_summaryChildren[0] = *m_model->append(m_rows.m_summary.children());
    m_rows.m_summaryChildren[0][m_modelColumns.m_item] = "Model";
    m_rows.m_summaryChildren[1] = *m_model->append(m_rows.m_summary.children());
    m_rows.m_summaryChildren[1][m_modelColumns.m_item] = "Serial Number";
    m_rows.m_summaryChildren[2] = *m_model->append(m_rows.m_summary.children());
    m_rows.m_summaryChildren[2][m_modelColumns.m_item] = "Number of target FPGAs";
    m_rows.m_summaryChildren[3] = *m_model->append(m_rows.m_summary.children());
    m_rows.m_summaryChildren[3][m_modelColumns.m_item] = "Number of clock generators";
    m_rows.m_summaryChildren[4] = *m_model->append(m_rows.m_summary.children());
    m_rows.m_summaryChildren[4][m_modelColumns.m_item] = "Number of DMA channels";
    m_rows.m_summaryChildren[5] = *m_model->append(m_rows.m_summary.children());
    m_rows.m_summaryChildren[5][m_modelColumns.m_item] = "Number of windows";
    m_rows.m_summaryChildren[6] = *m_model->append(m_rows.m_summary.children());
    m_rows.m_summaryChildren[6][m_modelColumns.m_item] = "Number of sensors";
    m_rows.m_summaryChildren[7] = *m_model->append(m_rows.m_summary.children());
    m_rows.m_summaryChildren[7][m_modelColumns.m_item] = "Number of I/O module sites";
    m_rows.m_summaryChildren[8] = *m_model->append(m_rows.m_summary.children());
    m_rows.m_summaryChildren[8][m_modelColumns.m_item] = "Number of Flash memory banks";
    m_rows.m_summaryChildren[9] = *m_model->append(m_rows.m_summary.children());
    m_rows.m_summaryChildren[9][m_modelColumns.m_item] = "Number of memory banks";
    m_rows.m_summaryChildren[10] = *m_model->append(m_rows.m_summary.children());
    m_rows.m_summaryChildren[10][m_modelColumns.m_item] = "Memory bank presence bitmap";
    m_rows.m_targetFpga = *(m_model->append());
    m_rows.m_targetFpga[m_modelColumns.m_item] = "Target FPGA";
    m_rows.m_window = *(m_model->append());
    m_rows.m_window[m_modelColumns.m_item] = "Window";
    m_rows.m_flash = *(m_model->append());
    m_rows.m_flash[m_modelColumns.m_item] = "Flash memory bank";
    m_rows.m_memory = *(m_model->append());
    m_rows.m_memory[m_modelColumns.m_item] = "Memory bank";
    m_rows.m_sensor = *(m_model->append());
    m_rows.m_sensor[m_modelColumns.m_item] = "Sensor";
    m_rows.m_module = *(m_model->append());
    m_rows.m_module[m_modelColumns.m_item] = "Module site";
    m_treeCtl.expand_row(Gtk::TreePath(m_rows.m_api), TRUE);
    m_treeCtl.expand_row(Gtk::TreePath(m_rows.m_summary), TRUE);
  }

  m_numDevice = numDevice;
}

// -------------------------------------------------------------------------
// End of IDeviceInfoWnd interface
// -------------------------------------------------------------------------

CDeviceInfoWnd::CDeviceInfoWnd(
  ISysMonWnd* pParent,
  bool_t bVisible) {

  m_pParent = pParent;
  m_hDevice = ADMXRC3_HANDLE_INVALID_VALUE;
  m_numDevice = 0;

  set_size_request(-1, 300);

  m_model = Gtk::TreeStore::create(m_modelColumns);
  m_treeCtl.set_model(m_model);
  m_treeCtl.set_headers_visible(FALSE);
  m_treeCtl.set_enable_tree_lines(TRUE);
  m_treeCtl.append_column("Property", m_modelColumns.m_item);
  m_treeCtl.append_column("Value", m_modelColumns.m_value);

  add(m_treeCtl);
  set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
}

CDeviceInfoWnd::~CDeviceInfoWnd() {
}

void
CDeviceInfoWnd::setProperty(
  Gtk::TreeRow& row,
  const std::ostringstream& item) {
  row[m_modelColumns.m_item] = item.str();
}

void
CDeviceInfoWnd::setValue(
  Gtk::TreeRow& row,
  const std::ostringstream& value) {
  row[m_modelColumns.m_value] = value.str();
}
