#pragma once

#include <objmodel.h>

class CDeviceInfoWnd : public Gtk::ScrolledWindow, public IDeviceInfoWnd
{
 public:
  CDeviceInfoWnd(ISysMonWnd* pParent, bool_t bVisible);
  virtual ~CDeviceInfoWnd();
  // IDeviceInfoWnd interface
  void deviceChanged();
  void numDeviceChanged();
  control_handle_t getNativeControl() { return *this; }
  ISysMonWnd* getParent() const { return m_pParent; }

protected:
  static const unsigned int m_maxNumFlashBank = 4;
  static const unsigned int m_maxNumMemoryBank = 16;
  static const unsigned int m_maxNumModuleSite = 4;
  static const unsigned int m_maxNumSensor = 64;
  static const unsigned int m_maxNumTargetFpga = 4;
  static const unsigned int m_maxNumWindow = 16;

  class ModelColumns : public Gtk::TreeModel::ColumnRecord {
  public:
    ModelColumns() {
      add(m_item);
      add(m_value);
    }

    Gtk::TreeModelColumn<Glib::ustring> m_item;
    Gtk::TreeModelColumn<Glib::ustring> m_value;
  };

  ISysMonWnd* m_pParent;
  ModelColumns m_modelColumns;
  Gtk::TreeView m_treeCtl;
  Glib::RefPtr<Gtk::TreeStore> m_model;
  unsigned int m_numDevice;
  unsigned int m_numTargetFpga;
  unsigned int m_numWindow;
  unsigned int m_numFlashBank;
  unsigned int m_numMemoryBank;
  unsigned int m_numModuleSite;
  unsigned int m_numSensor;
  ADMXRC3_HANDLE m_hDevice;
  struct {
    Gtk::TreeRow m_api;
    struct {
      Gtk::TreeRow m_api;
      Gtk::TreeRow m_driver;
    } m_apiChildren;
    Gtk::TreeRow m_summary;
    Gtk::TreeRow m_summaryChildren[11];
    Gtk::TreeRow m_targetFpga;
    struct {
      Gtk::TreeRow m_fpga;
      Gtk::TreeRow m_identifier;
      Gtk::TreeRow m_deviceCode;
      Gtk::TreeRow m_familyCode;
      Gtk::TreeRow m_subfamilyCode;
      Gtk::TreeRow m_package;
      Gtk::TreeRow m_device;
      Gtk::TreeRow m_packageCode;
      Gtk::TreeRow m_flags;
      struct {
        Gtk::TreeRow m_speed;
        Gtk::TreeRow m_stepping;
        Gtk::TreeRow m_notConfigurable;
      } m_flagsChildren;
      Gtk::TreeRow m_speedGrade;
      Gtk::TreeRow m_stepping;
      Gtk::TreeRow m_present;
    } m_targetFpgaChildren[m_maxNumTargetFpga];
    Gtk::TreeRow m_window;
    struct {
      Gtk::TreeRow m_window;
      Gtk::TreeRow m_busBase;
      Gtk::TreeRow m_busLimit;
      Gtk::TreeRow m_busSize;
      Gtk::TreeRow m_localBase;
      Gtk::TreeRow m_localLimit;
      Gtk::TreeRow m_localSize;
      Gtk::TreeRow m_virtualSize;
    } m_windowChildren[m_maxNumWindow];
    Gtk::TreeRow m_flash;
    struct {
      Gtk::TreeRow m_flash;
      Gtk::TreeRow m_identifier;
      Gtk::TreeRow m_size;
      Gtk::TreeRow m_useableStart;
      Gtk::TreeRow m_useableLimit;
      Gtk::TreeRow m_useableLength;
    } m_flashChildren[m_maxNumFlashBank];
    Gtk::TreeRow m_memory;
    struct {
      Gtk::TreeRow m_memory;
      Gtk::TreeRow m_maximumFrequency;
      Gtk::TreeRow m_minimumFrequency;
      Gtk::TreeRow m_physicalSize;
      Gtk::TreeRow m_physicalDataWidth;
      Gtk::TreeRow m_physicalEccWidth;
      Gtk::TreeRow m_physicalWidth;
      Gtk::TreeRow m_typeMask;
      struct {
        Gtk::TreeRow m_zbtFt;
        Gtk::TreeRow m_zbtP;
        Gtk::TreeRow m_sdramDdr;
        Gtk::TreeRow m_ssramDdr2;
        Gtk::TreeRow m_sdramSdr;
        Gtk::TreeRow m_sdramDdr2;
        Gtk::TreeRow m_ssramQdr;
        Gtk::TreeRow m_sdramDdr3;
      } m_typeChildren;
      Gtk::TreeRow m_connectivityMask;
      Gtk::TreeRow m_connectivityChildren[32];
      Gtk::TreeRow m_present;
    } m_memoryChildren[m_maxNumMemoryBank];
    Gtk::TreeRow m_sensor;
    struct {
      Gtk::TreeRow m_sensor;
      Gtk::TreeRow m_description;
      Gtk::TreeRow m_unit;
      Gtk::TreeRow m_datatype;
      Gtk::TreeRow m_exponent;
      Gtk::TreeRow m_error;
      Gtk::TreeRow m_capabilities;
      struct {
        Gtk::TreeRow m_swValue;
        Gtk::TreeRow m_hwMinimum;
        Gtk::TreeRow m_hwMaximum;
        Gtk::TreeRow m_hwAlarmLess;
        Gtk::TreeRow m_hwAlarmGreater;
      } m_capabilitiesChildren;
    } m_sensorChildren[m_maxNumSensor];
    Gtk::TreeRow m_module;
    struct {
      Gtk::TreeRow m_module;
      Gtk::TreeRow m_product;
      Gtk::TreeRow m_partNumber;
      Gtk::TreeRow m_manufacturer;
      Gtk::TreeRow m_serialNumber;
      Gtk::TreeRow m_ioVoltage;
      Gtk::TreeRow m_manufactureDate;
      Gtk::TreeRow m_flags;
      struct {
        Gtk::TreeRow m_legacyXrm;
        Gtk::TreeRow m_force2V5;
        Gtk::TreeRow m_romError;
        Gtk::TreeRow m_cSumError;
      } m_flagsItem;
      Gtk::TreeRow m_present;
    } m_moduleChildren[m_maxNumSensor];
  } m_rows;

protected:
  void setProperty(Gtk::TreeRow& row, const std::ostringstream& item);
  void setValue(Gtk::TreeRow& row, const std::ostringstream& item);
};
