#pragma once

#include <assert.h>
#include <stdint.h>
#include <stdarg.h>
#include <gtkmm.h>

#define ARRAY_LENGTH(x) (sizeof(x) / sizeof((x)[0]))

#define _T(x) x

#define RGB(r, g, b) (((r) << 0) | ((g) << 8) | ((b) << 16))
#define RED(x) (((x) >> 0) & 0xff)
#define GREEN(x) (((x) >> 8) & 0xff)
#define BLUE(x) (((x) >> 16) & 0xff)

#define ASSERT(x) do { if (!(x)) { throw new std::logic_error(#x); } } while (0)

using std::hex;
using std::dec;
using std::uppercase;
using std::nouppercase;
using std::fixed;

#define HEX hex << uppercase
#define DEC dec << nouppercase

typedef int bool_t;
typedef uint32_t color_t;
typedef char TCHAR;
typedef Gtk::Widget& control_handle_t;
typedef Gtk::Window& window_handle_t;

typedef int64_t timestamp_t;
#define TIMESTAMP_INVALID ((timestamp_t)-1)

__inline static timestamp_t
getTimeNow(void) {
  struct timespec ts;
  clock_gettime(CLOCK_REALTIME, &ts);
  return (int64_t)ts.tv_sec * 1000 + ts.tv_nsec / 1000000;
}

__inline static void
debugPrint(TCHAR* fmt, ...) {
  va_list ap;
  TCHAR buf[256];

  va_start(ap, fmt);
  vsnprintf(buf, ARRAY_LENGTH(buf), fmt, ap);
  va_end(ap);
  buf[ARRAY_LENGTH(buf) - 1] = _T('\0');
  fprintf(stderr, "%s", buf);
}
