#include "stdafx.h"
#include "afxwinappex.h"

#ifdef _DEBUG
# define new DEBUG_NEW
#endif

#include <platform.h>
#include <objmodel.h>

#include <math.h>

#include "scopewnd.h"

IMPLEMENT_DYNAMIC(CScopeWnd, CWnd)

BEGIN_MESSAGE_MAP(CScopeWnd, CWnd)
  ON_WM_CREATE()
  ON_WM_DESTROY()
  ON_WM_ERASEBKGND()
  ON_WM_MOUSELEAVE()
  ON_WM_MOUSEMOVE()
  ON_WM_PAINT()
  ON_WM_SIZE()
END_MESSAGE_MAP()

#if !defined(CLEAR_TRACES_ON_PERIOD_CHANGE)
# define CLEAR_TRACES_ON_PERIOD_CHANGE (0)
#endif

// -------------------------------------------------------------------------
// Start of IScopeWnd interface
// -------------------------------------------------------------------------

void
CScopeWnd::addTrace(
  unsigned int sensorIndex,
  int at)
{
  unsigned int i, pos, n = m_numTrace + 1;
  CTraceBuffer** ppNew;
  CTraceBuffer* pNew;
  RECT r;

  pos = (at == -1) ? m_numTrace : (unsigned int)at;
  ppNew = new CTraceBuffer*[n];
  ASSERT(NULL != ppNew);
  for (i = 0; i < pos; i++) {
    ppNew[i] = m_ppTraceBuffers[i];
  }
  pNew = new CTraceBuffer(sensorIndex);
  GetClientRect(&r);
  pNew->setSize(r.right);
  ppNew[pos] = pNew;
  for (i = pos + 1; i < n; i++) {
    ppNew[i] = m_ppTraceBuffers[i - 1];
  }
  delete[] m_ppTraceBuffers;
  m_ppTraceBuffers = ppNew;
  setNumTrace(n);
}

void
CScopeWnd::adjustScale(
  float value)
{
  bool_t bChanged = FALSE;

  if (value < m_minimum) {
    m_minimum = value;
    bChanged = TRUE;
  }
  if (value > m_maximum) {
    m_maximum = value;
    bChanged = TRUE;
  }
  if (bChanged) {
    calcYScaling(m_scaling, m_ppTraceBuffers, m_layout, m_numTrace);
    needRedraw();
  }
}

void
CScopeWnd::removeAllTraces()
{
  if (m_numTrace) {
    for (unsigned int i = 0; i < m_numTrace; i++) {
      delete m_ppTraceBuffers[i];
      m_ppTraceBuffers[i] = NULL;
    }
    delete[] m_ppTraceBuffers;
    m_ppTraceBuffers = NULL;
  }
  setNumTrace(0);
  m_minimum = 1.0e20f;
  m_maximum = -1.0e20f;
}

void
CScopeWnd::deviceChanged()
{
  removeAllTraces();
  needRedraw();
}

void
CScopeWnd::idle()
{
  if (m_bNeedRedraw) {
    Invalidate();
    m_bNeedRedraw = TRUE;
  }
}

void
CScopeWnd::update(timestamp_t timestamp)
{
  DeviceInfo devInfo = m_pParent->getParent()->getDeviceInfo();
  SensorInfo* pSensorInfo;
  CTraceBuffer* pTraceBuffer;
  unsigned int sensorIndex;
  CTraceBuffer::Point p;
  float point = 0.0f; // Prevent compiler whine

  if (m_numTrace) {
    p.m_timestamp = timestamp;
    p.m_bDiscontinuity = false;
    for (unsigned int i = 0; i < m_numTrace; i++) {
      pTraceBuffer = m_ppTraceBuffers[i];
      sensorIndex = pTraceBuffer->getSensorIndex();
      pSensorInfo = &devInfo.m_pSensors[sensorIndex];
      ASSERT(pSensorInfo->m_bKnownDataType);
      switch (pSensorInfo->m_info.DataType) {
      case ADMXRC3_DATA_BOOL:
        point = pSensorInfo->m_current.Bool ? 1.0f : 0.0f;
        break;

      case ADMXRC3_DATA_DOUBLE:
        point = (float)pSensorInfo->m_current.Double * pSensorInfo->m_exponent;
        break;

      case ADMXRC3_DATA_INT32:
        point = (float)pSensorInfo->m_current.Int32 * pSensorInfo->m_exponent;
        break;

      case ADMXRC3_DATA_UINT32:
        point = (float)pSensorInfo->m_current.UInt32 * pSensorInfo->m_exponent;
        break;

      default:
        ASSERT(FALSE);
        break;
      }
      p.m_value = point;
      pTraceBuffer->addPoint(p);
    }
    calcYScaling(m_scaling, m_ppTraceBuffers, m_layout, m_numTrace);
    needRedraw();
  }
}

void
CScopeWnd::updatePeriodChanged()
{
  if (m_numTrace) {
#if CLEAR_TRACES_ON_PERIOD_CHANGE
    for (unsigned int i = 0; i < m_numTrace; i++) {
      CTraceBuffer* pTraceBuffer = m_ppTraceBuffers[i];

      pTraceBuffer->clear();
    }
    calcYScaling(m_scaling, m_ppTraceBuffers, m_layout, m_numTrace);
#endif
    needRedraw();
  }
}

// -------------------------------------------------------------------------
// End of IScopeWnd interface
// -------------------------------------------------------------------------

CScopeWnd::CScopeWnd(
  IGraphWnd* pParent)
{
  const TCHAR* pWndClass = NULL;
  bool_t bSuccess;
  
  m_pParent = pParent;
  m_ppTraceBuffers = NULL;
  m_numTrace = 0;
  m_bMouseOver = FALSE;
  m_pPointBuffer = NULL;
  m_bNeedRedraw = FALSE;
  m_minimum = 1.0e20f;
  m_maximum = -1.0e20f;
  m_pPointBuffer = NULL;

  // Create POINT buffer for polyline drawing
  m_pPointBuffer = new POINT[m_pointBufferSize + 2];
  ASSERT(m_pPointBuffer != NULL);

  m_bgBrush.CreateSolidBrush(RGB(0, 0, 0));
  m_font.CreateStockObject(DEFAULT_GUI_FONT);

  pWndClass = AfxRegisterWndClass(0, ::LoadCursor(NULL, IDC_CROSS), (HBRUSH)m_bgBrush, NULL);
  bSuccess = CreateEx(0, pWndClass, _T(""), WS_VISIBLE | WS_CHILD, 0, 0, 1, 1, *dynamic_cast<CWnd*>(m_pParent), NULL, NULL);
  ASSERT(bSuccess);
}

CScopeWnd::~CScopeWnd()
{
}

void
CScopeWnd::needRedraw()
{
  if (m_bInvalidateOnlyWhenIdle) {
    m_bNeedRedraw = TRUE;
  } else {
    Invalidate();
  }
}

void
CScopeWnd::setNumTrace(
  unsigned int numTrace)
{
  RECT r;

  m_numTrace = numTrace;
  if (m_numTrace) {
    GetClientRect(&r);
    calcLayout(m_layout, r);
    calcYScaling(m_scaling, m_ppTraceBuffers, m_layout, m_numTrace);
  }
  m_pParent->getToolbar()->numTraceChanged();
  needRedraw();
}

void
CScopeWnd::calcLayout(
  Layout& layout,
  const RECT& clientRect)
{
  layout.m_axisYTop = clientRect.top + 15;
  layout.m_axisXYBottom = clientRect.bottom - 25;
  layout.m_axisXRight = clientRect.right - 20;
  layout.m_axisXYLeft = clientRect.left + 80;
  layout.m_axisYHeight = m_layout.m_axisXYBottom - m_layout.m_axisYTop;
  layout.m_axisXWidth = m_layout.m_axisXRight - m_layout.m_axisXYLeft;
}

void
CScopeWnd::calcYScaling(
  Scaling& /* scaling */,
  const CTraceBufferConstPtr* m_pTraceData,
  const Layout& /* layout */,
  unsigned int numTrace)
{
  float min = 1.0e20f, max = -1.0e20f;
  float delta, tmp;
  const CTraceBuffer* pTraceBuffer;

  if (m_minimum < min) {
    min = m_minimum;
  }
  if (m_maximum > max) {
    max = m_maximum;
  }

  ASSERT(numTrace > 0);
  m_scaling.m_bYValid = FALSE;
  for (unsigned int i = 0; i < numTrace; i++) {
    pTraceBuffer = m_pTraceData[i];
    if (pTraceBuffer->getLength()) {
      tmp = pTraceBuffer->getMin();
      if (tmp < min) {
        min = tmp;
      }
      tmp = pTraceBuffer->getMax();
      if (tmp > max) {
        max = tmp;
      }
      tmp = pTraceBuffer->getMean();
      if (tmp < min) {
        min = tmp;
      }
      if (tmp > max) {
        max = tmp;
      }
    }
  }

  delta = max - min;
  if (delta < 1.0e-9f) {
    return;
  }

  int maxYDivisions = (m_layout.m_axisYHeight + 1) / 20;
  float fl = log10f(delta / maxYDivisions);
  int exp = (int)floorf(fl);
  float tmp1 = fl - (float)exp;
  float tmp2 = powf(10, tmp1);
  float division;
  if (tmp2 < 2.001f) {
    division = 2.0f;
  } else if (tmp2 < 5.001f) {
    division = 5.0f;
  } else {
    division = 10.0f;
  }
  division = division * powf(10, (float)exp);
  int qMin = (int)floorf(min / division);
  int qMax = (int)ceilf(max / division - 0.01f);
  m_scaling.m_numYDivision = qMax - qMin;
  m_scaling.m_yDivision = division;
  m_scaling.m_yMin = (float)qMin * division;
  m_scaling.m_yMax = (float)qMax * division;
  m_scaling.m_yScaling = 1.0f / (m_scaling.m_yMax - m_scaling.m_yMin);
  m_scaling.m_bYValid = TRUE;
}

int
CScopeWnd::OnCreate(
  LPCREATESTRUCT lpCreateStruct)
{
  bool_t bSuccess;

  if (CWnd::OnCreate(lpCreateStruct) == -1) {
    return -1;
  }

  if (m_bDoubleBuffered) {
    bSuccess = m_memDC.CreateCompatibleDC(GetDC());
    ASSERT(bSuccess);
  }

  return 0;
}

void
CScopeWnd::OnDestroy()
{
  CWnd::OnDestroy();

  if (NULL != m_ppTraceBuffers) {
    for (unsigned int i = 0; i < m_numTrace; i++) {
      delete m_ppTraceBuffers[i];
      m_ppTraceBuffers[i] = NULL;
    }
    delete[] m_ppTraceBuffers;
    m_ppTraceBuffers = NULL;
  }
  if (m_bDoubleBuffered) {
    if (m_pOldBitmap != NULL) {
      m_memDC.SelectObject(m_pOldBitmap);
      m_pOldBitmap = NULL;
    }
  }
  if (m_pPointBuffer != NULL) {
    delete[] m_pPointBuffer;
    m_pPointBuffer = NULL;
  }
}

bool_t
CScopeWnd::OnEraseBkgnd(
  CDC* pDC)
{
  RECT r;
  GetClientRect(&r);
  CRect cr = CRect(r);
  if (m_bDoubleBuffered) {
    pDC = &m_memDC;
  }
  pDC->FillRect(&cr, &m_bgBrush);
  return TRUE;
}

void
CScopeWnd::OnMouseLeave()
{
  m_bMouseOver = FALSE;
  needRedraw();
}

void
CScopeWnd::OnMouseMove(
  UINT /* nFlags */,
  CPoint point)
{
  bool_t bSuccess;

  if (m_numTrace == 0) {
    return;
  }

  if (!m_bMouseOver) {
    memset(&m_mouseTrack, 0, sizeof(m_mouseTrack));
    m_mouseTrack.cbSize = sizeof(m_mouseTrack);
    m_mouseTrack.dwFlags = TME_LEAVE;
    m_mouseTrack.hwndTrack = (HWND)*this;
    m_mouseTrack.dwHoverTime = HOVER_DEFAULT;
    bSuccess = TrackMouseEvent(&m_mouseTrack);
    ASSERT(bSuccess);
    m_bMouseOver = TRUE;
  }
  m_mousePos.x = point.x;
  m_mousePos.y = point.y;
  needRedraw();
}

bool
CScopeWnd::xToTimestamp(
  int mouseX,
  timestamp_t t0,
  unsigned int updatePeriod,
  timestamp_t& timestamp)
{
  if (mouseX > m_layout.m_axisXYLeft && mouseX <= m_layout.m_axisXRight) {
    timestamp = t0 - (m_layout.m_axisXRight - mouseX) * (updatePeriod >> 1);
    return true;
  } else {
    return false;
  }
}

bool
CScopeWnd::timestampToX(
  timestamp_t timestamp,
  timestamp_t t0,
  unsigned int updatePeriod,
  int& x)
{
  float fx = (float)(t0 - timestamp) / (updatePeriod >> 1) + 0.5f;
  x = m_layout.m_axisXRight - (int)floorf(fx);
  return (x > m_layout.m_axisXYLeft && x <= m_layout.m_axisXRight);
}

bool
CScopeWnd::valueToY(
  float value,
  int& y)
{
  float fy = (value - m_scaling.m_yMin) * m_scaling.m_yScaling * (float)m_layout.m_axisYHeight;
  y = m_layout.m_axisXYBottom - (int)floorf(fy + 0.5f);
  return true;
}

void
CScopeWnd::drawAxes(
  CDC& dc)
{
  dc.SelectStockObject(WHITE_PEN);
  dc.MoveTo(m_layout.m_axisXYLeft, m_layout.m_axisYTop);
  dc.LineTo(m_layout.m_axisXYLeft, m_layout.m_axisXYBottom);
  dc.LineTo(m_layout.m_axisXRight, m_layout.m_axisXYBottom);
}

void
CScopeWnd::drawAxisLabels(
  CDC& dc,
  const SensorInfo* pSensorInfo)
{
  static const TCHAR* prefixStrs[] = {
    // From atto- (10^-18) up to exa- (10^15)
    _T("a"), _T("f"), _T("p"), _T("n"), _T("u"), _T("m"), _T(""), _T("k"), _T("M"), _T("G"), _T("T"), _T("P"), _T("E")
  };
  static const float factors[] = {
    1.0e18f, 1.0e15f, 1.0e12f, 1.0e9f, 1.0e6f, 1.0e3f, 1.0f, 1.0e-3f, 1.0e-6f, 1.0e-9f, 1.0e-12f, 1.0e-15f, 1.0e-18f
  };
  static const TCHAR* pOrderStr;
  static const TCHAR* pUnitStr;
  CString tmpStr;
  RECT r;
  float fy, fdy, val, logVal, factor;
  int y;

  dc.SetBkMode(TRANSPARENT);
  dc.SetTextColor(RGB(0xff, 0xff, 0xff));
  dc.SelectStockObject(WHITE_PEN);

  // Figure out unit mulitplier string ("k", "n", "G" etc.)
  val = m_scaling.m_yMax;
  logVal = log10f(fabs(m_scaling.m_yMax));
  int index = ((int)floorf(logVal + 0.5f) - 2) / 3 + 6;
  if (index < 0) {
    factor = 1.0e18f;
    index = 0;
  } else if (index > 12) {
    factor = 1.0e-18f;
    index = 12;
  } else {
    factor = factors[index];
  }
  pOrderStr = prefixStrs[index];

  // Get unit as a string
  switch (pSensorInfo->m_info.Unit) {
  case ADMXRC3_UNIT_A:
    pUnitStr = _T("A");
    break;

  case ADMXRC3_UNIT_V:
    pUnitStr = _T("V");
    break;

  case ADMXRC3_UNIT_C:
    pUnitStr = _T("C");
    break;

  case ADMXRC3_UNIT_HZ:
    pUnitStr = _T("Hz");
    break;

  case ADMXRC3_UNIT_RPM:
    pUnitStr = _T("RPM");
    break;

  default:
    pUnitStr = _T("");
    break;
  }

  // Draw unit with multiplier near top of Y-axis
  r.top = m_layout.m_axisYTop;
  r.bottom = m_layout.m_axisYTop + 20;
  r.right = 35;
  r.left = 10;
  tmpStr.Format(_T("%s%s"), pOrderStr, pUnitStr);
  dc.DrawText(tmpStr, -1, &r, DT_VCENTER | DT_RIGHT | DT_SINGLELINE);

  // Draw Y-axis labels
  fy = (float)m_layout.m_axisYTop;
  fdy = (float)m_layout.m_axisYHeight / (float)m_scaling.m_numYDivision;
  for (int i = 0; i <= m_scaling.m_numYDivision; i++) {
    y = (int)floorf(fy + 0.5f);
    dc.MoveTo(m_layout.m_axisXYLeft, y);
    dc.LineTo(m_layout.m_axisXYLeft - 5, y);
    tmpStr.Format(_T("%.1f"), (double)(val * factor));
    r.top = y - 8;
    r.bottom = y + 8;
    r.right = m_layout.m_axisXYLeft - 12;
    r.left = 37;
    dc.DrawText(tmpStr, -1, &r, DT_RIGHT | DT_SINGLELINE | DT_VCENTER);
    fy += fdy;
    val -= m_scaling.m_yDivision;
  }

  // Draw time unit right of X-axis
  GetClientRect(&r);
  r.top = m_layout.m_axisXYBottom - 10;
  r.bottom = m_layout.m_axisXYBottom + 10;
  r.left = m_layout.m_axisXRight + 5;
  dc.DrawText(_T("s"), -1, &r, DT_VCENTER | DT_LEFT | DT_SINGLELINE);

  // Draw X-axis labels and ticks
  timestamp_t t, t0 = m_pParent->getCurrentTimestamp();
  if (t0 != TIMESTAMP_INVALID) {
    const unsigned int minorPerMajor = 5;
    unsigned int updatePeriod = m_pParent->getParent()->getUpdatePeriod();
    unsigned int tickPeriodMajor = updatePeriod * 50;
    unsigned int tickPeriodMinor = tickPeriodMajor / minorPerMajor;
    unsigned int n;
    int x;
    GetClientRect(&r);
    r.top = m_layout.m_axisXYBottom + 5;
    ASSERT(t0 >= 0);
    t = t0 - (t0 % tickPeriodMinor);
    n = (int)(t % tickPeriodMajor) / tickPeriodMinor;
    timestampToX(t, t0, updatePeriod, x);
    while (x > m_layout.m_axisXYLeft - 4 && t >= 0) {
      if (0 == n) {
        tmpStr.Format(_T("%lld"), t / 1000);
        dc.MoveTo(x, r.top - 5);
        dc.LineTo(x, r.top);
        r.left = x - 25;
        r.right = x + 25;
        dc.DrawText(tmpStr, -1, &r, DT_CENTER | DT_SINGLELINE | DT_TOP);
        n = minorPerMajor;
      } else {
        dc.MoveTo(x, r.top - 5);
        dc.LineTo(x, r.top - 2);
      }
      n--;
      t -= tickPeriodMinor;
      timestampToX(t, t0, updatePeriod, x);
    }
  }
}

void
CScopeWnd::drawGuides(
  CDC& dc,
  const DeviceInfo& devInfo)
{
  SensorInfo* pSensorInfo;
  CTraceBuffer* pTraceBuffer;
  unsigned int sensorIndex;
  float point;
  int y;
  RECT r;
  POINT polygon[3];
  timestamp_t timestamp;

  dc.SelectObject(GetStockObject(DC_PEN));
  dc.SelectObject(GetStockObject(DC_BRUSH));

  GetClientRect(&r);
  dc.SetDCPenColor(RGB(0x40, 0x40, 0x40));
  dc.MoveTo(m_mousePos.x, r.top);
  dc.LineTo(m_mousePos.x, r.bottom);
  dc.MoveTo(r.left, m_mousePos.y);
  dc.LineTo(r.right, m_mousePos.y);

  if (m_scaling.m_bYValid && xToTimestamp(m_mousePos.x, m_pParent->getCurrentTimestamp(), m_pParent->getParent()->getUpdatePeriod(), timestamp)) {
    for (unsigned int i = 0; i < m_numTrace; i++) {
      pTraceBuffer = m_ppTraceBuffers[i];
      sensorIndex = pTraceBuffer->getSensorIndex();
      pSensorInfo = &devInfo.m_pSensors[sensorIndex];
      if (pTraceBuffer->getValueAt(timestamp, point)) {
        valueToY(point, y);
        dc.SetDCPenColor(pSensorInfo->m_traceColor);
        dc.SetDCBrushColor(pSensorInfo->m_traceColor);
        polygon[0].x = m_layout.m_axisXYLeft - 3;
        polygon[0].y = y;
        polygon[1].x = polygon[0].x - 7;
        polygon[1].y = y - 3;
        polygon[2].x = polygon[1].x;
        polygon[2].y = y + 3;
        dc.Polygon(polygon, 3);
      }
    }
  }
}

// Display a suitable message if there are no traces to display
void
CScopeWnd::drawMessage(
  CDC& dc,
  const DeviceInfo& devInfo)
{
  static const TCHAR* pMessageNoDevice = _T("No device selected\n\nPlease select a device.");
  static const TCHAR* pMessageNoSensor = _T("No sensors selected\n\nUse the toolbar buttons to\nadd sensors to the 'scope.");
  static const TCHAR* pMessageNoSensorAvail = _T("No sensors available\n\nPlease select a device\nwith some sensors.");
  static const TCHAR* pMessage;
  RECT r1, r2;

  dc.SetBkMode(TRANSPARENT);
  dc.SetTextColor(RGB(0xff, 0xff, 0xff));

  if (devInfo.m_hDevice == ADMXRC3_HANDLE_INVALID_VALUE) {
    pMessage = pMessageNoDevice;
  } else {
    if (devInfo.m_numAvailableSensor == 0) {
      pMessage = pMessageNoSensorAvail;
    } else {
      pMessage = pMessageNoSensor;
    }
  }
  GetClientRect(&r1);
  dc.DrawText(pMessage, -1, &r1, DT_CALCRECT);
  GetClientRect(&r2);
  r2.top = (r2.bottom - r1.bottom) >> 1;
  r2.bottom = r2.top + r1.bottom;
  r2.left = (r2.right - r1.right) >> 1;
  r2.right = r2.left + r1.right;
  dc.DrawText(pMessage, -1, &r2, DT_CENTER);
}

void
CScopeWnd::drawMinimalTicks(
  CDC& dc)
{
  /*
  ** We didn't manage to figure out scaling etc. for any traces
  ** so just draw the axes without labels.
  */
  dc.SelectStockObject(WHITE_PEN);
  dc.MoveTo(m_layout.m_axisXYLeft, m_layout.m_axisYTop);
  dc.LineTo(m_layout.m_axisXYLeft - 5, m_layout.m_axisYTop);
  dc.MoveTo(m_layout.m_axisXYLeft, m_layout.m_axisXYBottom);
  dc.LineTo(m_layout.m_axisXYLeft - 5, m_layout.m_axisXYBottom);
}

void
CScopeWnd::drawTrace(
  CDC& dc,
  SensorInfo* pSensorInfo,
  CTraceBuffer* pTraceBuffer,
  timestamp_t t0)
{
  POINT* pPoint;
  POINT* pPointLimit = m_pPointBuffer + m_pointBufferSize;
  unsigned int length;
  HPEN hDottedPen;
  LOGPEN logPen;
  unsigned int updatePeriod = m_pParent->getParent()->getUpdatePeriod();
  int xLow, xHigh;
  timestamp_t timestampLow, timestampHigh;
  unsigned int indexLow, indexHigh;
  timestamp_t minTimestamp, maxTimestamp;
  float valueLow, valueHigh;
  unsigned int index;
  int x1, x2;
  int y1, y2;

  // Skip this trace if it doesn't yet contain enough data points
  length = pTraceBuffer->getLength();
  if (length < 2) {
    return;
  }

  memset(&logPen, 0, sizeof(logPen));
  logPen.lopnColor = pSensorInfo->m_traceColor;
  logPen.lopnStyle = PS_DOT;
  logPen.lopnWidth.x = 1;
  logPen.lopnWidth.y = 0;
  hDottedPen = CreatePenIndirect(&logPen);

  dc.SelectObject(GetStockObject(DC_PEN));
  dc.SetDCPenColor(pSensorInfo->m_traceColor);

  xLow = m_layout.m_axisXYLeft + 1;
  xHigh = m_layout.m_axisXRight;
  xToTimestamp(xLow, t0, updatePeriod, timestampLow);
  xToTimestamp(xHigh, t0, updatePeriod, timestampHigh);
  ASSERT(xLow < xHigh);
  minTimestamp = pTraceBuffer->getMinTime();
  maxTimestamp = pTraceBuffer->getMaxTime();
  ASSERT(minTimestamp < maxTimestamp);
  if (minTimestamp <= timestampHigh && maxTimestamp >= timestampLow) {
    /* Dataset is partially or fully inside visible X-axis interval */
    if (minTimestamp < timestampLow) {
      pTraceBuffer->findTimeLE(timestampLow, indexLow);
      pTraceBuffer->getValueAt(timestampLow, valueLow);
    } else {
      indexLow = 0;
      timestampToX(minTimestamp, t0, updatePeriod, xLow);
      valueLow = pTraceBuffer->getValue(indexLow);
    }
    if (maxTimestamp > timestampHigh) {
      pTraceBuffer->findTimeGE(timestampLow, indexHigh);
      pTraceBuffer->getValueAt(timestampLow, valueHigh);
    } else {
      indexHigh = length - 1;
      timestampToX(maxTimestamp, t0, updatePeriod, xHigh);
      valueHigh = pTraceBuffer->getValue(indexHigh);
    }
    x1 = xLow;
    index = indexLow;
    valueToY(valueLow, y1);
    pPoint = m_pPointBuffer;
    pPoint->x = x1;
    pPoint->y = y1;
    pPoint++;
    while (index < indexHigh) {
      index++;
      const CTraceBuffer::Point& p = pTraceBuffer->getPoint(index);
      if (index == indexHigh) {
        x2 = xHigh;
        valueToY(valueHigh, y2);
      } else {
        timestampToX(p.m_timestamp, t0, updatePeriod, x2);
        valueToY(p.m_value, y2);
      }
      pPoint->x = x2;
      pPoint->y = y2;
      pPoint++;
      if (pPoint >= pPointLimit) {
        dc.Polyline(m_pPointBuffer, (int)(pPoint - m_pPointBuffer)); /* Cast is safe as buffer size is << maximum int */
        pPoint = m_pPointBuffer;
        pPoint->x = x2;
        pPoint->y = y2;
        pPoint++;
      }
      x1 = x2;
    }
    int nPoint = (int)(pPoint - m_pPointBuffer); /* Cast is safe as buffer size is << maximum int */
    if (nPoint >= 2) {
      dc.Polyline(m_pPointBuffer, nPoint);
    }
  } else {
    /* Dataset is wholly outside visible x-axis interval */
  }

  // Draw min and max
  dc.SelectObject(hDottedPen);
  valueToY(pTraceBuffer->getMin(), y1);
  dc.MoveTo(m_layout.m_axisXYLeft + 1, y1);
  dc.LineTo(m_layout.m_axisXRight, y1);
  valueToY(pTraceBuffer->getMax(), y2);
  dc.MoveTo(m_layout.m_axisXYLeft + 1, y2);
  dc.LineTo(m_layout.m_axisXRight, y2);

  dc.SelectObject(GetStockObject(DC_PEN));
  DeleteObject(hDottedPen);
}

void
CScopeWnd::OnPaint()
{
  ISysMonWnd* pMainWnd = m_pParent->getParent();
  const DeviceInfo& devInfo = pMainWnd->getDeviceInfo();
  CPaintDC dc(this);
  CDC* pDC;
  timestamp_t t0, minShow;

  if (m_bDoubleBuffered) {
    pDC = &m_memDC;
  } else {
    pDC = &dc;
  }
  pDC->SelectObject(m_font);
  if (m_numTrace) {
    if (m_bMouseOver) {
      drawGuides(*pDC, devInfo);
    }
    drawAxes(*pDC);
    if (!m_scaling.m_bYValid) {
      drawMinimalTicks(*pDC);
    } else {
      SensorInfo* pSensorInfo;
      CTraceBuffer* pTraceBuffer;
      unsigned int sensorIndex;

      /* Draw the Y-axis ticks and labels */
      pTraceBuffer = m_ppTraceBuffers[0];
      sensorIndex = pTraceBuffer->getSensorIndex();
      pSensorInfo = &devInfo.m_pSensors[sensorIndex];
      drawAxisLabels(*pDC, pSensorInfo);

      /*
      ** Now draw the traces
      */
      t0 = m_pParent->getCurrentTimestamp();
      xToTimestamp(m_layout.m_axisXYLeft + 1, t0, m_pParent->getParent()->getUpdatePeriod(), minShow);
      for (unsigned int i = 0; i < m_numTrace; i++) {
        pTraceBuffer = m_ppTraceBuffers[i];
        sensorIndex = pTraceBuffer->getSensorIndex();
        pSensorInfo = &devInfo.m_pSensors[sensorIndex];
        drawTrace(*pDC, pSensorInfo, pTraceBuffer, t0);
        if (i == 0 && pTraceBuffer->getLength() > 0) {
          timestamp_t min = pTraceBuffer->getMinTime();
          timestamp_t max = pTraceBuffer->getMaxTime();
          m_pParent->viewChanged(max - min, minShow - min, t0 - min);
        }
      }
    }
  } else {
    drawMessage(*pDC, devInfo);
  }

  if (m_bDoubleBuffered) {
    RECT r;
    GetClientRect(&r);
    dc.BitBlt(0, 0, r.right, r.bottom, &m_memDC, 0, 0, SRCCOPY);
  }
}

void
CScopeWnd::OnSize(
  UINT nType,
  int cx,
  int cy)
{
  RECT r = { 0, 0, cx, cy };
  bool_t bSuccess;
  
  CWnd::OnSize(nType, cx, cy);

  // Create offscreen bitmap for double-buffering
  if (m_bDoubleBuffered) {
    if (NULL != m_bitmap.GetSafeHandle()) {
      m_memDC.SelectObject(m_pOldBitmap);
      m_bitmap.DeleteObject();
      m_pOldBitmap = NULL;
    }
    bSuccess = m_bitmap.CreateCompatibleBitmap(GetDC(), cx, cy);
    ASSERT(bSuccess);
    m_pOldBitmap = m_memDC.SelectObject(&m_bitmap);
  }

  // Determine new layout of axes etc.
  if (m_numTrace) {
    for (unsigned int i = 0; i < m_numTrace; i++) {
      m_ppTraceBuffers[i]->setSize(cx);
    }
    calcLayout(m_layout, r);
    calcYScaling(m_scaling, m_ppTraceBuffers, m_layout, m_numTrace);
  }

  needRedraw();
}
