#include "stdafx.h"
#include "afxwinappex.h"

#ifdef _DEBUG
# define new DEBUG_NEW
#endif

#include <platform.h>
#include <objmodel.h>

#include "control_id.h"
#include "sensorinfownd.h"

IMPLEMENT_DYNAMIC(CSensorInfoWnd, CWnd)

BEGIN_MESSAGE_MAP(CSensorInfoWnd, CWnd)
    ON_NOTIFY(LVN_GETINFOTIP, IDCTL_SENSORINFO_LIST, OnGetInfoTip)
    ON_WM_CREATE()
    ON_WM_CLOSE()
    ON_WM_SIZE()
END_MESSAGE_MAP()

static const TCHAR* g_prefixStr[] = {
  // From atto- (10^-18) up to exa- (10^15)
  _T("a"), _T("f"), _T("p"), _T("n"), _T("u"), _T("m"), _T(""), _T("k"), _T("M"), _T("G"), _T("T"), _T("P"), _T("E")
};
static const float g_factor[] = {
  1.0e18f, 1.0e15f, 1.0e12f, 1.0e9f, 1.0e6f, 1.0e3f, 1.0f, 1.0e-3f, 1.0e-6f, 1.0e-9f, 1.0e-12f, 1.0e-15f, 1.0e-18f
};

static const float g_fMinTotalWidth = 580;
static const float g_fColumnIndex = 0.07f;
static const float g_fColumnData = 0.155f;
static const float g_fColumnUnit = 0.155f;
static const float g_fColumnDescription = 1.0f - g_fColumnIndex - g_fColumnData - g_fColumnUnit;

// -------------------------------------------------------------------------
// Start of ISensorInfoWnd interface
// -------------------------------------------------------------------------

void
CSensorInfoWnd::deviceChanged()
{
  SensorInfo* pSensorInfo;
  CString str;

  m_listCtl.SetRedraw(FALSE);
  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  if (devInfo.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE) {
    m_listCtl.DeleteAllItems();
  }
  if (ADMXRC3_HANDLE_INVALID_VALUE != devInfo.m_hDevice) {
    m_listCtl.InsertItem(0, _T("#"));
    m_listCtl.SetItem(0, 1, LVIF_TEXT, _T("Description"), 0, 0, 0, (LPARAM)0);
    m_listCtl.SetItem(0, 2, LVIF_TEXT, _T("Value"), 0, 0, 0, (LPARAM)0);
    m_listCtl.SetItem(0, 3, LVIF_TEXT, _T("Unit"), 0, 0, 0, (LPARAM)0);
    for (unsigned int i = 0; i < devInfo.m_numSensor; i++) {
      unsigned int j = i + 1;
      int exponent, prefix;
      const TCHAR* pPrefixStr = _T("");
      const TCHAR* pUnitStr;

      pSensorInfo = &devInfo.m_pSensors[i];
      str.Format(_T("%u"), i);
      m_listCtl.InsertItem(j, str);
      if (pSensorInfo->m_bValid) {
        m_listCtl.SetItem(j, 1, LVIF_TEXT, pSensorInfo->m_info.Description, 0, 0, 0, (LPARAM)0);
	if (pSensorInfo->m_info.Unit != ADMXRC3_UNIT_NONE) {
	  exponent = pSensorInfo->m_info.Exponent;
	  prefix = (exponent + 18) / 3;
	  if (prefix < 0 || prefix > (int)ARRAY_LENGTH(g_prefixStr)) {
	    pPrefixStr = _T("?");
	  } else {
	    pPrefixStr = g_prefixStr[prefix];
	  }
	}
        pUnitStr = CAdmxrc3Help::getUnitString(pSensorInfo->m_info.Unit);
        if (NULL == pUnitStr) {
          pUnitStr = _T("?");
        }
        str.Format(_T("%s%s"), pPrefixStr, pUnitStr);
        m_listCtl.SetItem(j, 3, LVIF_TEXT, str, 0, 0, 0, (LPARAM)0);
      } else {
        m_listCtl.SetItem(j, 1, LVIF_TEXT, _T("(Unable to determine)"), 0, 0, 0, (LPARAM)0);
      }
    }
  }
  m_listCtl.SetRedraw(TRUE);
}

void
CSensorInfoWnd::numDeviceChanged()
{
  unsigned int numDevice = m_pParent->getDeviceInfo().m_numDevice;

  if ((m_numDevice == 0) == (numDevice == 0)) {
    return;
  }

  /* Nothing to do yet */

  m_numDevice = numDevice;
}

void
CSensorInfoWnd::update()
{
  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  SensorInfo* pSensorInfo;
  CString str;
  unsigned int n = devInfo.m_numSensor;
  double doubleVal;
  INT32 int32Val;
  UINT32 uint32Val;

  m_listCtl.SetRedraw(FALSE);
  for (unsigned int i = 0; i < n; i++) {
    pSensorInfo = &devInfo.m_pSensors[i];
    if (pSensorInfo->m_bValid) {
      switch (pSensorInfo->m_info.DataType) {
      case ADMXRC3_DATA_BOOL:
        m_listCtl.SetItem(i + 1, 2, LVIF_TEXT, pSensorInfo->m_current.Bool ? _T("TRUE") : _T("FALSE"), 0, 0, 0, (LPARAM)0);
        break;

      case ADMXRC3_DATA_DOUBLE:
        doubleVal = pSensorInfo->m_current.Double;
        str.Format(_T("%.3f"), doubleVal);
        m_listCtl.SetItem(i + 1, 2, LVIF_TEXT, str, 0, 0, 0, (LPARAM)0);
        break;

      case ADMXRC3_DATA_INT32:
        int32Val = pSensorInfo->m_current.Int32;
        str.Format(_T("%ld"), (long)int32Val);
        m_listCtl.SetItem(i + 1, 2, LVIF_TEXT, str, 0, 0, 0, (LPARAM)0);
        break;

      case ADMXRC3_DATA_UINT32:
        uint32Val = pSensorInfo->m_current.UInt32;
        str.Format(_T("%lu"), (unsigned long)uint32Val);
        m_listCtl.SetItem(i + 1, 2, LVIF_TEXT, str, 0, 0, 0, (LPARAM)0);
        break;

      default:
        m_listCtl.SetItem(i + 1, 2, LVIF_TEXT, _T("?"), 0, 0, 0, (LPARAM)0);
        break;
      }
    }
  }
  m_listCtl.SetRedraw(TRUE);
}

// -------------------------------------------------------------------------
// End of ISensorInfoWnd interface
// -------------------------------------------------------------------------

CSensorInfoWnd::CSensorInfoWnd(
  ISysMonWnd* pParent,
  bool_t bVisible)
{
  const TCHAR* pWndClass = NULL;
  bool_t bSuccess;

  m_pParent = pParent;
  m_numDevice = 0;

  m_bgBrush.CreateSysColorBrush(COLOR_BTNFACE);
  m_font.CreateStockObject(DEFAULT_GUI_FONT);

  pWndClass = AfxRegisterWndClass(0, ::LoadCursor(NULL, IDC_ARROW), (HBRUSH)m_bgBrush, NULL);
  bSuccess = CreateEx(WS_EX_CONTROLPARENT, pWndClass, _T(""), WS_CHILD | (bVisible ? WS_VISIBLE : 0), 0, 0, 1, 1, *dynamic_cast<CWnd*>(m_pParent), NULL, NULL);
  ASSERT(bSuccess);
}

CSensorInfoWnd::~CSensorInfoWnd()
{
}

int
CSensorInfoWnd::OnCreate(
  LPCREATESTRUCT lpCreateStruct)
{
  RECT r = { 0, 0, 0, 0 };

  if (CWnd::OnCreate(lpCreateStruct) == -1) {
    return -1;
  }

  m_listCtl.CreateEx(WS_EX_STATICEDGE, WS_CHILD | WS_VISIBLE | WS_TABSTOP | LVS_REPORT | LVS_NOCOLUMNHEADER, r, this, IDCTL_SENSORINFO_LIST);
  m_listCtl.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_INFOTIP);
  m_listCtl.SetFont(&m_font);
  m_listCtl.InsertColumn(0, _T("#"), LVCFMT_LEFT, (int)(g_fMinTotalWidth * g_fColumnIndex), -1);
  m_listCtl.InsertColumn(1, _T("Description"), LVCFMT_LEFT, (int)(g_fMinTotalWidth * g_fColumnDescription), -1);
  m_listCtl.InsertColumn(2, _T("Value"), LVCFMT_LEFT, (int)(g_fMinTotalWidth * g_fColumnData), -1);
  m_listCtl.InsertColumn(3, _T("Unit"), LVCFMT_LEFT, (int)(g_fMinTotalWidth * g_fColumnUnit), -1);

  return 0;
}

void
CSensorInfoWnd::OnGetInfoTip(
  NMHDR* /* pNotifyStruct */,
  LRESULT* /* pResult */)
{
}

void
CSensorInfoWnd::OnSize(
  UINT nType,
  int cx,
  int cy)
{
  CWnd::OnSize(nType, cx, cy);

  m_listCtl.MoveWindow(0, 0, cx, cy);
}
