#pragma once

#include <objmodel.h>

#ifdef _DEBUG
# define new DEBUG_NEW
#endif

class CSysMonWnd : public CWnd, public ISysMonWnd
{
	DECLARE_DYNAMIC(CSysMonWnd)

public:
	CSysMonWnd();
	virtual ~CSysMonWnd();
  // ISysMonWnd interface
  bool_t addGraphWnd(void);
  const DeviceInfo& getDeviceInfo() const { return m_device; }
  IDeviceInfoWnd* getDeviceInfoWnd() const { return m_pDeviceInfoWnd; }
  IGraphWnd* getGraphWnd(unsigned int index) const {
    ASSERT(index < m_numGraphWnd);
    return m_ppGraphWnd[index];
  }
  window_handle_t getNativeWindow() { return *this; }
  unsigned int getNumGraphWnd() const { return m_numGraphWnd; }
  unsigned int getUpdatePeriod() const { return m_updatePeriods[m_updatePeriodIndex]; }
  ISensorInfoWnd* getSensorInfoWnd() const { return m_pSensorInfoWnd; }
  void idle();
  bool_t removeGraphWnd(unsigned int index);

protected:
  // Following members define main window default and minimum size, in pixels
  static const int m_defaultWidth = 640;
  static const int m_defaultHeight = 400;
  static const int m_minWidth = 640;
  static const int m_minHeight = 400;

  // Timer ID used for periodic sensor reads
  static const UINT_PTR m_timerId = 1U;
  // Sensor read periods, selected m_freqSelectCtl
  static const UINT m_updatePeriods[];

  unsigned int m_deviceIndex;
  unsigned int m_numGraphWnd;
  int m_currentTab;
  int m_updatePeriodIndex;
  bool_t m_bLogging;
  bool_t m_bLogAppend;
  CString m_logFilename;
  timestamp_t m_logTimestamp;
  CMenu m_trayMenu;
  CMenu* m_pSystemMenu;
  CTabCtrl m_tabCtl;
  CFont m_font;
  CBrush m_bgBrush;
  CStatic m_deviceLabelCtl;
  CComboBox m_deviceSelectCtl;
  CStatic m_freqLabelCtl;
  CComboBox m_freqSelectCtl;
  IGraphWnd** m_ppGraphWnd;
  IDeviceInfoWnd* m_pDeviceInfoWnd;
  ISensorInfoWnd* m_pSensorInfoWnd;
  DeviceInfo m_device;
  HICON m_hIcon;
  HICON m_hIconTray;
  bool_t m_bTrayIconShown;
  bool_t m_bTrayErrorShown;
  NOTIFYICONDATA m_trayIconData;

protected:
  void enumerateDevices(ADMXRC3_CARD_INFOEX* pCardInfoEx);
  virtual bool_t PreTranslateMessage(MSG* pMsg);
  void readSensors(timestamp_t timestamp);
  bool_t removeGraphWnd(IGraphWnd* pGraphWnd);
  bool_t setDevice(unsigned int deviceIndex, bool_t bFromControl = FALSE);
  bool_t unsetDevice();
  void setNumGraphWnd(unsigned int n);
  bool_t showTrayIcon(bool_t bShow);
  bool_t showTrayError(bool_t bShow, const TCHAR* pTitle, const TCHAR* pMessage);
  LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
  void startLogging(const CString& filename, bool bAppend);
  void stopLogging(bool_t bQuiet);
  void emitLogHeader(FILE* pFile, const DeviceInfo& devInfo, timestamp_t& timestamp);

  DECLARE_MESSAGE_MAP()

protected:
  afx_msg void OnClose();
  afx_msg void OnClickedFinish();
  afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
  afx_msg BOOL OnDeviceChange(UINT nEventType, DWORD_PTR eventData);
  afx_msg void OnSelectDevice();
  afx_msg void OnSelectUpdatePeriod();
  afx_msg void OnSize(UINT nType, int cx, int cy);
  afx_msg void OnSizing(UINT nSide, RECT* pRect);
  afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
  afx_msg void OnTabChange(NMHDR* pNotifyStruct, LRESULT* pResult);
  afx_msg void OnTimer(UINT_PTR nTimerId); 
  afx_msg void OnTrayExit(); 
  afx_msg void OnTrayRestore(); 
  afx_msg void OnAbout();
  afx_msg void OnStartNewLog();
  afx_msg void OnAppendToLog();
  afx_msg void OnStopLog();
};
