--==============================================================================
--
--     File Name: adb3_target_pkg.vhd
--       Version: 0.4
--          Date: 25 Jul 2011
--         Model: Synthesisable
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: Package adb3_target_pkg
--
--   Description: Package of components used in target FPGAs using ADB3.
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2009-2011 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 06 Nov 2009: Initial design
--                0.2, TW,  09 Feb 2011:
--                      - Moved some component declarations into new
--                        adb3_ocp_comp package.
--                0.3, KDC, 08 Jun 2011: Updated mptl_if_target_wrap component
--                0.4, KDC, 25 Jul 2011:
--                     - Moved functions to adb3_ocp_comp_pkg: ds_base_conv,
--                       ds_mask_conv, dma_base_conv, dma_mask_conv, mask_vec_width
--                     - Removed component declarations: mptl_if_target_sim,
--                       mptl64par_interface_target_6tl_top, mptl64par_interface_target_6tl,
--                       mptl128_interface_target_6t1_top, mptl128_interface_target_6t1
--
--==============================================================================

--
-- Exported functions
--
--   make_defined_s2m
--   make_defined
--
--
-- Exported components:
--
--   mptl_if_target_wrap
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_inc_pkg.all;

package adb3_target_pkg is

  function make_defined_s2m(
    constant x : in adb3_ocp_s2mT)
  return adb3_ocp_s2mT;

  function make_defined(
    constant x : in std_logic_vector)
  return std_logic_vector;


  component mptl_if_target_wrap
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
    dma_abort                : in    std_logic_vector(DMA_CHANNELS-1 downto 0);
    -- MPTL Interface
    mptl_b2t                 : in    mptl_b2t_t;
    mptl_t2b                 : out   mptl_t2b_t;
    mptl_clk                 : in    std_logic_dbl_t;
    -- MPTL Initialisation
    ocp_ready                : in    std_logic;
    mptl_sb_b2t              : in    mptl_sb_b2t_t;
    mptl_sb_t2b              : out   mptl_sb_t2b_t;
    -- MPTL Sideband
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

end;


package body adb3_target_pkg is

  -- Make sure OCP s2m record has defined data for MPTL serial interface simulation
  function make_defined_s2m(
    constant x : in adb3_ocp_s2mT)
  return adb3_ocp_s2mT is
    variable m : adb3_ocp_s2mT;  -- New OCP record
  begin
    m.CmdAccept  := x.CmdAccept;
    m.DataAccept := x.DataAccept;
    m.Resp       := x.Resp;
    m.Data       := make_defined(x.Data);
    m.Tag        := make_defined(x.Tag);
    return m;
  end;

  -- Convert any non '0' or '1' bits of std_logic_vector to '0'
  function make_defined(
    constant x : in std_logic_vector)
  return std_logic_vector is
    variable n : natural := x'length;               -- Size of x in bits (min size of 1)
    variable m : std_logic_vector(n - 1 downto 0);  -- New vector of size n bits
  begin
    for i in 0 to (n - 1) loop
      if (x(i) = '0')or(x(i) = '1') then
        m(i) := x(i);
      else
        m(i) := '0';
      end if;
    end loop;

    return m;
  end;

end;