--==============================================================================
--
--     File Name: adb3_target_tb_pkg.vhd
--       Version: 0.1
--          Date: 29 Aug 2011
--         Model: Simulation only
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: Package adb3_target_tb_pkg
--
--   Description: Package of components used in testbenches using MPTL.
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2011 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 29 Aug 2011: Initial design
--
--==============================================================================

--
-- Exported components:
--
--   mptl_if_bridge_wrap
--   test_board_clks
--

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_inc_pkg.all;
use work.adb3_target_tb_inc_pkg.all;

package adb3_target_tb_pkg is

  component mptl_if_bridge_wrap
  port(
    -- OCP Interface
    ocp_clk_in               : in    std_logic;
    direct_slave_m2s         : in    adb3_ocp_m2sT;
    direct_slave_s2m         : out   adb3_ocp_s2mT;
    dma_channels_m2s         : in    adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
    dma_channels_s2m         : out   adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
    direct_masters_m2s       : out   adb3_ocp_m2sT_array(0 to DM_CHANNELS-1);
    direct_masters_s2m       : in    adb3_ocp_s2mT_array(0 to DM_CHANNELS-1);
    dma_abort                : out   std_logic_vector(DMA_CHANNELS-1 downto 0);
    ocp_clk_out              : out   std_logic;
    -- MPTL Interface
    mptl_t2b                 : in    mptl_t2b_t;
    mptl_b2t                 : out   mptl_b2t_t;
    mptl_clk                 : in    std_logic_dbl_t;
    -- MPTL Initialisation
    mptl_sb_t2b              : in    mptl_sb_t2b_t;
    mptl_sb_b2t              : out   mptl_sb_b2t_t;
    mptl_online              : out   std_logic;
    -- MPTL Sideband
    gpio_b2t                 : in    std_logic_vector(15 downto 0);
    gpio_t2b                 : out   std_logic_vector(15 downto 0);
    mptl_status              : out   std_logic_vector(63 downto 0);
    mptl_config              : in    std_logic_vector(31 downto 0));
  end component;

  component test_board_clks
  generic(
    clks_out_freq : clks_out_exp_t);
  port(
    -- Target clock interface
    target_clks_in     : out   clks_in_t;
    target_clks_mgt_in : out   clks_mgt_in_t;
    target_clks_out    : in    clks_out_t;
    target_ref_clk     : out   std_logic_dbl_t;
    target_mptl_clk    : out   std_logic_dbl_t;
    target_pcie_clk    : out   std_logic_dbl_t;
    -- Bridge clock interface
    bridge_lclk_freq   : in    real;
    bridge_ocp_clk     : out   std_logic;
    bridge_mptl_clk    : out   std_logic_dbl_t;
    -- Test interface
    clks_out_pass      : out   boolean);
  end component;

end;


package body adb3_target_tb_pkg is

end;
