--==============================================================================
--
--     File Name: adb3_target_tb_pkg_pcie.vhd
--       Version: 0.1
--          Date: 29 Aug 2011
--         Model: Simulation only
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: Package adb3_target_tb_pkg
--
--   Description: Package of components used in testbenches using ADB3 PCIe
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2011 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 29 Aug 2011: Initial design
--
--==============================================================================

--
-- Exported components:
--
--   pcie_if_host_wrap
--   test_board_clks
--

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_inc_pkg.all;
use work.adb3_target_tb_inc_pkg.all;

package adb3_target_tb_pkg is

  component pcie_if_host_wrap
  port(
    -- OCP Interface
    direct_slave_m2s   : in    adb3_ocp_m2sT;
    direct_slave_s2m   : out   adb3_ocp_s2mT;
    dma_channels_m2s   : in    adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
    dma_channels_s2m   : out   adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
    direct_masters_m2s : out   adb3_ocp_m2sT_array(0 to DM_CHANNELS-1);
    direct_masters_s2m : in    adb3_ocp_s2mT_array(0 to DM_CHANNELS-1);
    dma_abort          : out   std_logic_vector(DMA_CHANNELS-1 downto 0);
    interrupt          : out   std_logic;
    ocp_clk_out        : out   std_logic;
    -- PCIe Interface
    pcie_t2h           : in    pcie_t2h_t;
    pcie_h2t           : out   pcie_h2t_t);
  end component;

  component test_board_clks
  generic(
    clks_out_freq : clks_out_exp_t);
  port(
    -- Target clock interface
    target_clks_in     : out   clks_in_t;
    target_clks_mgt_in : out   clks_mgt_in_t;
    target_clks_out    : in    clks_out_t;
    target_ref_clk     : out   std_logic_dbl_t;
    target_mptl_clk    : out   std_logic_dbl_t;
    target_pcie_clk    : out   std_logic_dbl_t;
    -- Bridge clock interface
    bridge_lclk_freq   : in    real;
    bridge_ocp_clk     : out   std_logic;
    bridge_mptl_clk    : out   std_logic_dbl_t;
    -- Test interface
    clks_out_pass      : out   boolean);
  end component;

end;


package body adb3_target_tb_pkg is

end;
