--==============================================================================
--
--     File Name: adb3_target_types_pkg.vhd
--       Version: 0.3
--          Date: 25 Jul 2011
--         Model: Synthesisable
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: Package adb3_target_types_pkg
--
--   Description: Package of components used in target FPGAs using ADB3.
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2009-2011 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 22 Feb 2009: Initial design
--                0.2, KDC, 08 Jun 2011:
--                       - Added ADM_XRC_6TGE to board_type_t
--                       - Increased MAX_PN6_GPIO_WIDTH to 46
--                0.3, KDC, 25 Jul 2011:
--                     - Removed board_type_t. Board in use is now selected by
--                       inclusion of appropriate VHDL design files.
--                     - Changed MAX_PN4_GPIO_WIDTH to MAX_GPIO_DE_WIDTH
--                     - Changed MAX_PN6_GPIO_WIDTH to MAX_GPIO_SE_WIDTH
--                     - Added MAX_PCIE_SER_WIDTH
--                     - Changed MAX_MEM_BANKS to 7
--                     - Added MAX_GPIO_FR_WIDTH
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;

package adb3_target_types_pkg is

  -- Define type for design use
  type target_use_t is (SIM_OCP,SIM_MPTL,SYN_NGC);

  -- Define maximum number of OCP Channels
  constant MAX_DS_CHANNELS  : natural := 1;
  constant MAX_DMA_CHANNELS : natural := 4;
  constant MAX_DM_CHANNELS  : natural := 3;

  -- Define maximum number of memory banks
  constant MAX_MEM_BANKS : natural := 7;

  -- Define maximum width of serial data interfaces
  constant MAX_MPTL_SER_WIDTH : natural := 8;
  constant MAX_PCIE_SER_WIDTH : natural := 8;

  -- Define maximum width of GPIO interfaces
  constant MAX_GPIO_FR_WIDTH : natural := 146;
  constant MAX_GPIO_SE_WIDTH : natural := 64;
  constant MAX_GPIO_DE_WIDTH : natural := 32;

end;


package body adb3_target_types_pkg is

end;