--==============================================================================
--
--     File Name: test_board_clks_6t1.vhd
--       Version: 0.1
--          Date: 02 Jun 2011
--         Model: Simulation only
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: ADB3 target design test clock generation
--
--   Description: Testbench clock generation/test for target FPGA
--
--   Limitations: ADM-XRC-6T1 testbench
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2011 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 02 Jun 2011: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_target_inc_pkg.all;
use work.adb3_target_tb_inc_pkg.all;

entity test_board_clks is
generic(
  clks_out_freq : clks_out_exp_t);
port(
  -- Target clock interface
  target_clks_in     : out   clks_in_t;
  target_clks_mgt_in : out   clks_mgt_in_t;
  target_clks_out    : in    clks_out_t;
  target_ref_clk     : out   std_logic_dbl_t;
  target_mptl_clk    : out   std_logic_dbl_t;
  target_pcie_clk    : out   std_logic_dbl_t;
  -- Bridge clock interface
  bridge_lclk_freq   : in    real;
  bridge_ocp_clk     : out   std_logic;
  bridge_mptl_clk    : out   std_logic_dbl_t;
  -- Test interface
  clks_out_pass      : out   boolean);
end;

architecture sim_6t1 of test_board_clks is

  -- Default value for programmable clock generated by bridge FPGA
  constant BRIDGE_LCLK_PER_INIT : time := 5 ns; -- 200 MHz (5 MHz to 700 MHz)

  -- Programmable reference clocks
  signal mgt113_clk0 : std_logic_dbl_t := ('1','0');
  signal mgt113_clk1 : std_logic_dbl_t := ('1','0');
  signal mgt114_clk0 : std_logic_dbl_t := ('1','0');
  signal mgt115_clk0 : std_logic_dbl_t := ('1','0');
  signal mgt116_clk0 : std_logic_dbl_t := ('1','0');
  signal mgt117_clk0 : std_logic_dbl_t := ('1','0');

  -- Fixed reference clocks
  signal bridge_ref_clk_int : std_logic_dbl_t := ('1','0');
  signal target_ref_clk_int : std_logic_dbl_t := ('1','0');

  -- Connector reference clocks
  signal target_xrm_clkin : std_logic_dbl_t := ('1','0');

  -- Bridge reference clocks
  signal target_lclk_per : time := BRIDGE_LCLK_PER_INIT;
  signal target_lclk : std_logic_dbl_t := ('1','0');

  -- MPTL clocks
  signal bridge_mptl_clk_int : std_logic_dbl_t := ('1','0');

  -- target_clks_out frequency measurement
  signal xrm_mgtclk_p_pass : boolean;
  signal xrm_mgtclk_n_pass : boolean;

begin

  ---------------------------
  -- Generate board clocks --
  ---------------------------

  -- Target MGT clocks
  mgt113_clk0.p <= not mgt113_clk0.p after MGT113_CLK0_PER / 2;
  mgt113_clk0.n <= not mgt113_clk0.n after MGT113_CLK0_PER / 2;

  mgt113_clk1.p <= not mgt113_clk1.p after MGT113_CLK1_PER / 2;
  mgt113_clk1.n <= not mgt113_clk1.n after MGT113_CLK1_PER / 2;

  mgt114_clk0.p <= not mgt114_clk0.p after MGT114_CLK0_PER / 2;
  mgt114_clk0.n <= not mgt114_clk0.n after MGT114_CLK0_PER / 2;

  mgt115_clk0.p <= not mgt115_clk0.p after MGT115_CLK0_PER / 2;
  mgt115_clk0.n <= not mgt115_clk0.n after MGT115_CLK0_PER / 2;

  mgt116_clk0.p <= not mgt116_clk0.p after MGT116_CLK0_PER / 2;
  mgt116_clk0.n <= not mgt116_clk0.n after MGT116_CLK0_PER / 2;

  mgt117_clk0.p <= not mgt117_clk0.p after MGT117_CLK0_PER / 2;
  mgt117_clk0.n <= not mgt117_clk0.n after MGT117_CLK0_PER / 2;

  -- Reference clocks
  target_ref_clk_int.p <= not target_ref_clk_int.p after TARGET_REF_CLK_PER / 2;
  target_ref_clk_int.n <= not target_ref_clk_int.n after TARGET_REF_CLK_PER / 2;

  bridge_ref_clk_int.p <= not bridge_ref_clk_int.p after BRIDGE_REF_CLK_PER / 2;
  bridge_ref_clk_int.n <= not bridge_ref_clk_int.n after BRIDGE_REF_CLK_PER / 2;

  -- XRM2 Connector clocks
  target_xrm_clkin.p <= not target_xrm_clkin.p after TARGET_XRM_CLKIN_PER / 2;
  target_xrm_clkin.n <= not target_xrm_clkin.n after TARGET_XRM_CLKIN_PER / 2;

  -- Bridge generated clocks
  target_lclk_per <= (1 sec / bridge_lclk_freq) when (bridge_lclk_freq > 0.0) else BRIDGE_LCLK_PER_INIT;

  target_lclk.p <= not target_lclk.p after target_lclk_per / 2;
  target_lclk.n <= not target_lclk.n after target_lclk_per / 2;

  -- MPTL clocks
  bridge_mptl_clk_int.p <= not bridge_mptl_clk_int.p after BRIDGE_MPTL_CLK_PER / 2;
  bridge_mptl_clk_int.n <= not bridge_mptl_clk_int.n after BRIDGE_MPTL_CLK_PER / 2;


  ----------------------------
  -- Generate target clocks --
  ----------------------------

  -- Connect clock inputs
  target_clks_in.ref_clk      <= target_ref_clk_int;
  target_clks_in.lclk         <= target_lclk;
  target_clks_in.xrm_gclk_m2c <= target_xrm_clkin;

  -- Connect MGT clock inputs
  target_clks_mgt_in.mgt113_clk0 <= mgt113_clk0; -- Osc. Reference clock 3 250 MHz
  target_clks_mgt_in.mgt113_clk1 <= mgt113_clk1; -- PCIe Reference clock 1 250 MHz
  target_clks_mgt_in.mgt114_clk0 <= mgt114_clk0; -- PCIe Reference clock 2 250 MHz
  target_clks_mgt_in.mgt115_clk0 <= mgt115_clk0; -- Osc. Reference clock 2 250 MHz (MPTL clock)
  target_clks_mgt_in.mgt116_clk0 <= mgt116_clk0; -- Osc. Reference clock 1 250 MHz
  target_clks_mgt_in.mgt117_clk0 <= mgt117_clk0; -- XRM  Reference clock (variable)

  -- Reference clock for designs that don't use target_clks_in
  target_ref_clk <= target_ref_clk_int;

  -- MPTL clock for designs that don't use target_clks_mgt_in
  target_mptl_clk <= mgt115_clk0;

  -- PCIe clock for designs that don't use target_clks_mgt_in
  target_pcie_clk <= mgt113_clk1;


  ----------------------------
  -- Generate bridge clocks --
  ----------------------------

  -- OCP clock for full MPTL simulation
  bridge_ocp_clk <= bridge_ref_clk_int.p;

  -- MPTL clock for full MPTL simulation
  bridge_mptl_clk <= bridge_mptl_clk_int;


  ------------------------
  -- Test target clocks --
  ------------------------

  -- Determine target_clks_out frequencies
  target_clks_in_period_p : process(target_clks_out)
    variable xrm_mgtclk_p_rising : time := 0 ns;
    variable xrm_mgtclk_n_rising : time := 0 ns;
    variable xrm_mgtclk_p_period : time := 0 ns;
    variable xrm_mgtclk_n_period : time := 0 ns;
  begin
    if rising_edge(target_clks_out.xrm_mgtclk_c2m.p) then
      xrm_mgtclk_p_period := now - xrm_mgtclk_p_rising;
      xrm_mgtclk_p_rising := now;

      if (xrm_mgtclk_p_period = (1 us / clks_out_freq.xrm_mgtclk_c2m)) then
        xrm_mgtclk_p_pass <= true;
      else
        xrm_mgtclk_p_pass <= false;
      end if;
    end if;

    if rising_edge(target_clks_out.xrm_mgtclk_c2m.n) then
      xrm_mgtclk_n_period := now - xrm_mgtclk_n_rising;
      xrm_mgtclk_n_rising := now;

      if (xrm_mgtclk_n_period = (1 us / clks_out_freq.xrm_mgtclk_c2m)) then
        xrm_mgtclk_n_pass <= true;
      else
        xrm_mgtclk_n_pass <= false;
      end if;
    end if;
  end process;

  clks_out_pass <= xrm_mgtclk_p_pass and xrm_mgtclk_n_pass;

end;