--==============================================================================
--
--     File Name: adb3_target_tb_inc_pkg_6tadv8.vhd
--       Version: 0.1
--          Date: 19 Aug 2011
--         Model: Simulation only
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: Package adb3_target_inc_pkg
--
--   Description: Package used in target testbench
--
--   Limitations: ADM-XRC-6TADV8 testbench
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2011 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 19 Jul 2011: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.ddr3_sdram_pkg.all;
use work.adb3_target_types_pkg.all;
use work.adb3_target_inc_pkg.all;


package adb3_target_tb_inc_pkg is

  constant BOARD_TYPE : string(1 to 14) := "ADM-XRC-6TADV8";

  -- Target FPGA clock outputs expected frequency
  type clks_out_exp_t is record
    unused : natural;
  end record;

  -- DDR3 component models
  constant DDR3_1G_PART : part_t := MT41J64M16_15E;
  constant DDR3_2G_PART : part_t := MT41J128M16_15E;
  constant DDR3_4G_PART : part_t := MT41J256M16_15E;

  -------------------
  -- Build options --
  -------------------

  -- Option z: Target device
  -- Option y: Target device speed

  -- Option m: DDR3 Memory size
  -- 1/2/4 Gib DDR3 SDRAM parts fitted
  constant OPTION_M : part_t := DDR3_1G_PART;

  -- Option c: Cooling

  -- Option p: Pn6 Connector
  -- True:     Pn6 connector fitted
  -- False: No Pn6 connector fitted
  constant OPTION_P : boolean := false; -- unused


  ------------------------------
  -- Clock period definitions --
  ------------------------------

  constant CLK_666_PER : time :=  1.50 ns;
  constant CLK_250_PER : time :=  4.00 ns;
  constant CLK_200_PER : time :=  5.00 ns;
  constant CLK_156_PER : time :=  6.40 ns;
  constant CLK_125_PER : time :=  8.00 ns;
  constant CLK_100_PER : time := 10.00 ns;


  -- Board clocks
  -- P5 PCIe reference = 100MHz
  constant PCIE_REFCLK : time := CLK_100_PER; -- Target clock

  -- U23 input 156.25MHz 25ppm oscillator
  constant MGTCLK156A : time := CLK_156_PER; -- Target clock
  constant MGTCLK156B : time := CLK_156_PER; -- Target clock
  constant MGTCLK156C : time := CLK_156_PER; -- Target clock
  constant MGTCLK156D : time := CLK_156_PER; -- Target clock

  -- U28 input 200MHz 25ppm oscillator
  constant REFCLK200M : time := CLK_200_PER; -- Target reference clock


  -- Target clocks
  constant TARGET_REF_CLK_PER : time := REFCLK200M;

  -- Target MGT clocks
  constant MGT110_CLK0_PER : time := MGTCLK156D;  -- Reference clock 156.25MHz
  constant MGT112_CLK0_PER : time := PCIE_REFCLK; -- PCIe Reference clock 100 MHz
  constant MGT113_CLK0_PER : time := MGTCLK156C;  -- Reference clock 156.25MHz
  constant MGT115_CLK0_PER : time := MGTCLK156B;  -- Reference clock 156.25MHz
  constant MGT117_CLK0_PER : time := MGTCLK156A;  -- Reference clock 156.25MHz


  ---------------------
  -- On-board memory --
  ---------------------

  -- Select DDR3 part for board simulation
  constant BOARD_DDR3_PART : part_t := OPTION_M;

  -- DDR3 SDRAM address widths
  constant BOARD_DDR3_BANK_ROW_WIDTH : natural := BOARD_DDR3_PART.part_size.arow_width;
  constant BOARD_DDR3_BANK_COL_WIDTH : natural := BOARD_DDR3_PART.part_size.acol_width;
  constant BOARD_DDR3_BANK_BNK_WIDTH : natural := BOARD_DDR3_PART.part_size.bank_width;

  -- DDR3 SDRAM address width = Bank+Row+Col and convert to byte addressing (+2)
  constant BOARD_DDR3_BYTE_ADDR_WIDTH : natural := BOARD_DDR3_BANK_BNK_WIDTH + BOARD_DDR3_BANK_ROW_WIDTH + BOARD_DDR3_BANK_COL_WIDTH + 2;

  -- Address width for each on-board memory bank
  constant BOARD_MEM_BYTE_ADDR_WIDTH_ARRAY : mem_byte_addr_width_array_t := ((DDR3_BANKS+DDR3_BANK0-1) downto DDR3_BANK0 => BOARD_DDR3_BYTE_ADDR_WIDTH);

end;


package body adb3_target_tb_inc_pkg is

end;