--==============================================================================
--
--     File Name: pcie_if_host_wrap_sim_6tadv8.vhd
--       Version: 0.1
--          Date: 19 Jul 2011
--         Model: Simulation only
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: Host PCIe interface wrapper
--
--   Description: Instantiate host PCIe interface OCP-only simulation code
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2011 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 19 Jul 2011: Initial version
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_inc_pkg.all;

entity pcie_if_host_wrap is
port(
  -- OCP Interface
  direct_slave_m2s   : in    adb3_ocp_m2sT;
  direct_slave_s2m   : out   adb3_ocp_s2mT;
  dma_channels_m2s   : in    adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
  dma_channels_s2m   : out   adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
  direct_masters_m2s : out   adb3_ocp_m2sT_array(0 to DM_CHANNELS-1);
  direct_masters_s2m : in    adb3_ocp_s2mT_array(0 to DM_CHANNELS-1);
  dma_abort          : out   std_logic_vector(DMA_CHANNELS-1 downto 0);
  interrupt          : out   std_logic;
  ocp_clk_out        : out   std_logic;
  -- PCIe Interface
  pcie_t2h           : in    pcie_t2h_t;
  pcie_h2t           : out   pcie_h2t_t);
end;


architecture sim_6tadv8 of pcie_if_host_wrap is

  signal target_ocp_clk : std_logic;

begin

  -- Simulation interface inputs
  target_ocp_clk     <= pcie_t2h.target_ocp_clk;
  direct_slave_s2m   <= pcie_t2h.direct_slave_s2m;
  dma_channels_s2m   <= pcie_t2h.dma_channels_s2m;
  direct_masters_m2s <= pcie_t2h.direct_masters_m2s;
  dma_abort          <= pcie_t2h.dma_abort;
  interrupt          <= pcie_t2h.interrupt;

  -- Simulation interface outputs
  pcie_h2t.direct_slave_m2s   <= direct_slave_m2s;
  pcie_h2t.dma_channels_m2s   <= dma_channels_m2s;
  pcie_h2t.direct_masters_s2m <= direct_masters_s2m;


  -- Direct OCP-OCP simulation
  -- Host ocp clock must be target ocp clock
  ocp_clk_out <= target_ocp_clk;

end;