--==============================================================================
--
--     File Name: pcie_if_target_wrap_6tadv8.vhd
--       Version: 0.1
--          Date: 18 Jul 2011
--         Model: Synthesisable
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: Target PCIe interface wrapper
--
--   Description: Instantiate full-PCIe simulation netlist or synthesis
--                core for target PCIe interface
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2011 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 18 Jul 2011: Initial version
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;

Library UNISIM;
use UNISIM.vcomponents.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_inc_pkg.all;

entity pcie_if_target_wrap is
port(
  ref_clk          : in    std_logic;
  -- OCP Interface
  ocp_clk          : in    std_logic;
  direct_slave_m2s : out   adb3_ocp_m2sT;
  direct_slave_s2m : in    adb3_ocp_s2mT;
  dma_channels_m2s : out   adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
  dma_channels_s2m : in    adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
  dma_abort        : in    std_logic_vector(DMA_CHANNELS-1 downto 0);
  interrupt        : in    std_logic;
  -- PCIe Interface
  pcie_rst_l       : in    std_logic;
  pcie_clk         : in    std_logic_dbl_t;
  pcie_h2t         : in    pcie_h2t_t;
  pcie_t2h         : out   pcie_t2h_t;
  -- Board IO
  model_in         : in    adb3_model_in_t;
  model_out        : out   adb3_model_out_t;
  model_inout      : inout adb3_model_inout_t);
end;


architecture syn_6tadv8 of pcie_if_target_wrap is

  -- Unused IO
  signal ref_clk_out : std_logic;
  signal ocp_clk_out : std_logic;

  -- Buffered PCIE clock
  signal pcie_clk_ibuf : std_logic;

  -- Internal OCP
  signal ds_int_m2s  : adb3_ocp_m2sT;
  signal ds_int_s2m  : adb3_ocp_s2mT;
  signal dma_int_m2s : adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
  signal dma_int_s2m : adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);

  -- dma_abort is currently missing from these components

  component admxrc6tadv8_pcie_x4 is
  port (
    -- Reset and Clocking
    sys_reset_n      : in    std_logic;
    sys_clk          : in    std_logic;
    refclkout        : out   std_logic;
    ocpclkout        : out   std_logic;
    ocpclkin         : in    std_logic;
    rclkin           : in    std_logic;
    -- PCIE
    pci_exp_txn      : out   std_logic_vector(3 downto 0);
    pci_exp_txp      : out   std_logic_vector(3 downto 0);
    pci_exp_rxn      : in    std_logic_vector(3 downto 0);
    pci_exp_rxp      : in    std_logic_vector(3 downto 0);
    -- Flash and other board IO
    model_in         : in    adb3_model_in_t;
    model_inout      : inout adb3_model_inout_t;
    model_out        : out   adb3_model_out_t;
    -- OCP interface to User Logic
    direct_slave_m2s : out   adb3_ocp_m2sT;
    direct_slave_s2m : in    adb3_ocp_s2mT;
    dma_channels_m2s : out   adb3_ocp_m2sT_array(0 to 1);
    dma_channels_s2m : in    adb3_ocp_s2mT_array(0 to 1);
    interrupt        : in    std_logic);
  end component;

  component admxrc6tadv8_pcie_x8 is
  port (
    -- Reset and Clocking
    sys_reset_n      : in    std_logic;
    sys_clk          : in    std_logic;
    refclkout        : out   std_logic;
    ocpclkout        : out   std_logic;
    ocpclkin         : in    std_logic;
    rclkin           : in    std_logic;
    -- PCIE
    pci_exp_txn      : out   std_logic_vector(7 downto 0);
    pci_exp_txp      : out   std_logic_vector(7 downto 0);
    pci_exp_rxn      : in    std_logic_vector(7 downto 0);
    pci_exp_rxp      : in    std_logic_vector(7 downto 0);
    -- Flash and other board IO
    model_in         : in    adb3_model_in_t;
    model_inout      : inout adb3_model_inout_t;
    model_out        : out   adb3_model_out_t;
    -- OCP interface to User Logic
    direct_slave_m2s : out   adb3_ocp_m2sT;
    direct_slave_s2m : in    adb3_ocp_s2mT;
    dma_channels_m2s : out   adb3_ocp_m2sT_array(0 to 1);
    dma_channels_s2m : in    adb3_ocp_s2mT_array(0 to 1);
    interrupt        : in    std_logic);
  end component;

begin

  -- Extract MGT clock to pcie_clk
  pcie_clk_ibufds_gtxe1_i : IBUFDS_GTXE1
  port map(
    I     => pcie_clk.p,
    IB    => pcie_clk.n,
    CEB   => '0',
    O     => pcie_clk_ibuf,
    ODIV2 => open);


  direct_slave_m2s <= ds_int_m2s;
  ds_int_s2m       <= direct_slave_s2m;

  dma_channels_m2s <= dma_int_m2s;
  dma_int_s2m      <= dma_channels_s2m;

  -- dma_abort is currently missing from these component instantiations

  gen2x4_g : if (PCIE_SER_WIDTH = 4) generate
    -- PCIe simulation netlist or synthesis .NGC core for PCIe gen2x4
    admxrc6tadv8_pcie_x4_i : admxrc6tadv8_pcie_x4
      port map (
        -- Reset and Clocking
        sys_reset_n      => pcie_rst_l,    -- in    std_logic
        sys_clk          => pcie_clk_ibuf, -- in    std_logic
        refclkout        => ref_clk_out,   -- out   std_logic
        ocpclkout        => ocp_clk_out,   -- out   std_logic
        ocpclkin         => ocp_clk,       -- in    std_logic
        rclkin           => ref_clk,       -- in    std_logic
        -- PCIe
        pci_exp_txn      => pcie_t2h.n,    -- out   std_logic_vector(3 downto 0)
        pci_exp_txp      => pcie_t2h.p,    -- out   std_logic_vector(3 downto 0)
        pci_exp_rxn      => pcie_h2t.n,    -- in    std_logic_vector(3 downto 0)
        pci_exp_rxp      => pcie_h2t.p,    -- in    std_logic_vector(3 downto 0)
        -- Flash and other board IO
        model_in         => model_in,      -- in    adb3_model_in_t
        model_inout      => model_inout,   -- inout adb3_model_inout_t
        model_out        => model_out,     -- out   adb3_model_out_t
        -- OCP interface to User Logic
        direct_slave_m2s => ds_int_m2s,    -- out   adb3_ocp_m2sT
        direct_slave_s2m => ds_int_s2m,    -- in    adb3_ocp_s2mT
        dma_channels_m2s => dma_int_m2s,   -- out   adb3_ocp_m2sT_array(0 to 1)
        dma_channels_s2m => dma_int_s2m,   -- in    adb3_ocp_s2mT_array(0 to 1)
        interrupt        => interrupt);    -- in    std_logic
  end generate;

  gen2x8_g : if (PCIE_SER_WIDTH = 8) generate
    -- PCIe simulation netlist or synthesis .NGC core for PCIe gen2x8
    admxrc6tadv8_pcie_x8_i : admxrc6tadv8_pcie_x8
      port map (
        -- Reset and Clocking
        sys_reset_n      => pcie_rst_l,    -- in    std_logic
        sys_clk          => pcie_clk_ibuf, -- in    std_logic
        refclkout        => ref_clk_out,   -- out   std_logic
        ocpclkout        => ocp_clk_out,   -- out   std_logic
        ocpclkin         => ocp_clk,       -- in    std_logic
        rclkin           => ref_clk,       -- in    std_logic
        -- PCIe
        pci_exp_txn      => pcie_t2h.n,    -- out   std_logic_vector(7 downto 0)
        pci_exp_txp      => pcie_t2h.p,    -- out   std_logic_vector(7 downto 0)
        pci_exp_rxn      => pcie_h2t.n,    -- in    std_logic_vector(7 downto 0)
        pci_exp_rxp      => pcie_h2t.p,    -- in    std_logic_vector(7 downto 0)
        -- Flash and other board IO
        model_in         => model_in,      -- in    adb3_model_in_t
        model_inout      => model_inout,   -- inout adb3_model_inout_t
        model_out        => model_out,     -- out   adb3_model_out_t
        -- OCP interface to User Logic
        direct_slave_m2s => ds_int_m2s,    -- out   adb3_ocp_m2sT
        direct_slave_s2m => ds_int_s2m,    -- in    adb3_ocp_s2mT
        dma_channels_m2s => dma_int_m2s,   -- out   adb3_ocp_m2sT_array(0 to 1)
        dma_channels_s2m => dma_int_s2m,   -- in    adb3_ocp_s2mT_array(0 to 1)
        interrupt        => interrupt);    -- in    std_logic
  end generate;

end;