--==============================================================================
--
--     File Name: pcie_if_target_wrap_sim_6tadv8.vhd
--       Version: 0.1
--          Date: 18 Jul 2011
--         Model: Simulation only
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: Target PCIe interface wrapper
--
--   Description: Instantiate target PCIe interface OCP-only simulation code
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2011 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 18 Jul 2011: Initial version
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_inc_pkg.all;

entity pcie_if_target_wrap is
port(
  ref_clk          : in    std_logic;
  -- OCP Interface
  ocp_clk          : in    std_logic;
  direct_slave_m2s : out   adb3_ocp_m2sT;
  direct_slave_s2m : in    adb3_ocp_s2mT;
  dma_channels_m2s : out   adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
  dma_channels_s2m : in    adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
  dma_abort        : in    std_logic_vector(DMA_CHANNELS-1 downto 0);
  interrupt        : in    std_logic;
  -- PCIe Interface
  pcie_rst_l       : in    std_logic;
  pcie_clk         : in    std_logic_dbl_t;
  pcie_h2t         : in    pcie_h2t_t;
  pcie_t2h         : out   pcie_t2h_t;
  -- Board IO
  model_in         : in    adb3_model_in_t;
  model_out        : out   adb3_model_out_t;
  model_inout      : inout adb3_model_inout_t);
end;


architecture sim_6tadv8 of pcie_if_target_wrap is

  -- Unused IO
  signal direct_masters_m2s : adb3_ocp_m2sT_array(0 to DM_CHANNELS-1) := (others => ADB3_OCP_DEFAULT_M2S);
  signal direct_masters_s2m : adb3_ocp_s2mT_array(0 to DM_CHANNELS-1);

begin

  -- Simulation interface inputs
  direct_slave_m2s   <= pcie_h2t.direct_slave_m2s;
  dma_channels_m2s   <= pcie_h2t.dma_channels_m2s;
  direct_masters_s2m <= pcie_h2t.direct_masters_s2m;

  -- Simulation interface outputs
  -- dma_abort output uses MPTL
  pcie_t2h.target_ocp_clk     <= ocp_clk;
  pcie_t2h.direct_slave_s2m   <= direct_slave_s2m;
  pcie_t2h.dma_channels_s2m   <= dma_channels_s2m;
  pcie_t2h.direct_masters_m2s <= direct_masters_m2s;
  pcie_t2h.dma_abort          <= dma_abort;
  pcie_t2h.interrupt          <= interrupt;

  -- ADV8 specific board interface
  model_out   <= (others => '0');
  model_inout <= (fl_a      => (others => 'Z'),
                  fl_sel_n  => 'Z',
                  fl_oe_n   => 'Z',
                  fl_we_n   => 'Z',
                  fl_rst_n  => 'Z',
                  fl_d      => (others => 'Z'),
                  fl_adv_n  => 'Z',
                  fl_clk    => 'Z',
                  fl_wait_n => 'Z',
                  iic0_scl  => 'Z',
                  iic0_sda  => 'Z',
                  iic1_scl  => 'Z',
                  iic1_sda  => 'Z');

end;