--==============================================================================
--
--     File Name: test_board_clks_6tadv8.vhd
--       Version: 0.1
--          Date: 19 Aug 2011
--         Model: Simulation only
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: ADB3 target design test clock generation
--
--   Description: Testbench clock generation/test for target FPGA
--
--   Limitations: ADM-XRC-6TADV8 testbench
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2011 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 19 Aug 2011: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_target_inc_pkg.all;
use work.adb3_target_tb_inc_pkg.all;

entity test_board_clks is
generic(
  clks_out_freq : clks_out_exp_t);
port(
  -- Target clock interface
  target_clks_in     : out   clks_in_t;
  target_clks_mgt_in : out   clks_mgt_in_t;
  target_clks_out    : in    clks_out_t;
  target_ref_clk     : out   std_logic_dbl_t;
  target_mptl_clk    : out   std_logic_dbl_t;
  target_pcie_clk    : out   std_logic_dbl_t;
  -- Bridge clock interface
  bridge_lclk_freq   : in    real;
  bridge_ocp_clk     : out   std_logic;
  bridge_mptl_clk    : out   std_logic_dbl_t;
  -- Test interface
  clks_out_pass      : out   boolean);
end;

architecture sim_adv8 of test_board_clks is

  -- Programmable reference clocks
  signal mgt110_clk0 : std_logic_dbl_t := ('1','0');
  signal mgt112_clk0 : std_logic_dbl_t := ('1','0');
  signal mgt113_clk0 : std_logic_dbl_t := ('1','0');
  signal mgt115_clk0 : std_logic_dbl_t := ('1','0');
  signal mgt117_clk0 : std_logic_dbl_t := ('1','0');

  -- Fixed reference clocks
  signal target_ref_clk_int : std_logic_dbl_t := ('1','0');

  -- target_clks_out frequency measurement

begin

  ---------------------------
  -- Generate board clocks --
  ---------------------------

  -- Target MGT clocks
  mgt110_clk0.p <= not mgt110_clk0.p after MGT110_CLK0_PER / 2;
  mgt110_clk0.n <= not mgt110_clk0.n after MGT110_CLK0_PER / 2;

  mgt112_clk0.p <= not mgt112_clk0.p after MGT112_CLK0_PER / 2;
  mgt112_clk0.n <= not mgt112_clk0.n after MGT112_CLK0_PER / 2;

  mgt113_clk0.p <= not mgt113_clk0.p after MGT113_CLK0_PER / 2;
  mgt113_clk0.n <= not mgt113_clk0.n after MGT113_CLK0_PER / 2;

  mgt115_clk0.p <= not mgt115_clk0.p after MGT115_CLK0_PER / 2;
  mgt115_clk0.n <= not mgt115_clk0.n after MGT115_CLK0_PER / 2;

  mgt117_clk0.p <= not mgt117_clk0.p after MGT117_CLK0_PER / 2;
  mgt117_clk0.n <= not mgt117_clk0.n after MGT117_CLK0_PER / 2;

  -- Reference clocks
  target_ref_clk_int.p <= not target_ref_clk_int.p after TARGET_REF_CLK_PER / 2;
  target_ref_clk_int.n <= not target_ref_clk_int.n after TARGET_REF_CLK_PER / 2;


  ----------------------------
  -- Generate target clocks --
  ----------------------------

  -- Connect clock inputs
  target_clks_in.ref_clk <= target_ref_clk_int;

  -- Connect MGT clock inputs
  target_clks_mgt_in.mgt110_clk0 <= mgt110_clk0; -- Reference clock 156.25MHz
  target_clks_mgt_in.mgt112_clk0 <= mgt112_clk0; -- PCIe Reference clock 100 MHz
  target_clks_mgt_in.mgt113_clk0 <= mgt113_clk0; -- Reference clock 156.25MHz
  target_clks_mgt_in.mgt115_clk0 <= mgt115_clk0; -- Reference clock 156.25MHz
  target_clks_mgt_in.mgt117_clk0 <= mgt117_clk0; -- Reference clock 156.25MHz

  -- Reference clock for designs that don't use target_clks_in
  target_ref_clk <= target_ref_clk_int;

  -- MPTL clock for designs that don't use target_clks_mgt_in
  target_mptl_clk <= ('U','U');

  -- PCIe clock for designs that don't use target_clks_mgt_in
  target_pcie_clk <= mgt112_clk0;


  ----------------------------
  -- Generate bridge clocks --
  ----------------------------

  -- OCP clock for full MPTL simulation
  bridge_ocp_clk <= 'U';

  -- MPTL clock for full MPTL simulation
  bridge_mptl_clk <= ('U','U');


  ------------------------
  -- Test target clocks --
  ------------------------

  clks_out_pass <= true;

end;