--==============================================================================
--
--     File Name: mptl_if_bridge_wrap_6tge.vhd
--       Version: 0.1
--          Date: 18 Jul 2011
--         Model: Simulation only
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: Bridge MPTL interface wrapper
--
--   Description: Instantiate full-MPTL simulation netlist for bridge
--                MPTL interface
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2011 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 18 Jul 2011:
--                     - ADM-XRC-6TGE specific version of mptl_if_bridge_wrap.
--                       moved from hdl/vhdl/common/mptl
--
--==============================================================================


library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_types_pkg.all;
use work.adb3_target_inc_pkg.all;
use work.adb3_target_pkg.all;

entity mptl_if_bridge_wrap is
port(
  -- OCP Interface
  ocp_clk_in               : in    std_logic;
  direct_slave_m2s         : in    adb3_ocp_m2sT;
  direct_slave_s2m         : out   adb3_ocp_s2mT;
  dma_channels_m2s         : in    adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
  dma_channels_s2m         : out   adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
  direct_masters_m2s       : out   adb3_ocp_m2sT_array(0 to DM_CHANNELS-1);
  direct_masters_s2m       : in    adb3_ocp_s2mT_array(0 to DM_CHANNELS-1);
  dma_abort                : out   std_logic_vector(DMA_CHANNELS-1 downto 0);
  ocp_clk_out              : out   std_logic;
  -- MPTL Interface
  mptl_t2b                 : in    mptl_t2b_t;
  mptl_b2t                 : out   mptl_b2t_t;
  mptl_clk                 : in    std_logic_dbl_t;
  -- MPTL Initialisation
  mptl_sb_t2b              : in    mptl_sb_t2b_t;
  mptl_sb_b2t              : out   mptl_sb_b2t_t;
  mptl_online              : out   std_logic;
  -- MPTL Sideband
  gpio_b2t                 : in    std_logic_vector(15 downto 0);
  gpio_t2b                 : out   std_logic_vector(15 downto 0);
  mptl_status              : out   std_logic_vector(63 downto 0);
  mptl_config              : in    std_logic_vector(31 downto 0));
end;


architecture sim_6tge of mptl_if_bridge_wrap is

  signal dm_int_m2s : adb3_ocp_m2sT_array(0 to 2);
  signal dm_int_s2m : adb3_ocp_s2mT_array(0 to 2);

  signal target_configured : std_logic;
  signal target_gtp_online : std_logic;
  signal bridge_gtp_online : std_logic;

  component mptl128_interface_bridge_6t1
  port(
    -- OCP Interface
    ocp_clk            : in  std_logic;
    direct_slave_m2s   : in  adb3_ocp_m2sT;
    direct_slave_s2m   : out adb3_ocp_s2mT;
    dma_channels_m2s   : in  adb3_ocp_m2sT_array(0 to 3);
    dma_channels_s2m   : out adb3_ocp_s2mT_array(0 to 3);
    direct_masters_m2s : out adb3_ocp_m2sT_array(0 to 2);
    direct_masters_s2m : in  adb3_ocp_s2mT_array(0 to 2);
    dma_abort          : out std_logic_vector(3 downto 0);
    -- MPTL Interface
    mptl_tx            : out mptl_pins_t;
    mptl_rx            : in  mptl_pins_t;
    mptl_clk           : in  std_logic_dbl_t;
    -- Bridge to Target Sideband Signals
    target_configured  : in  std_logic;
    target_gtp_online  : in  std_logic;
    bridge_gtp_online  : out std_logic;
    -- MPTL64 Sideband Interface
    gpio_in            : in  std_logic_vector(15 downto 0);
    gpio_out           : out std_logic_vector(15 downto 0);
    mptl_status        : out std_logic_vector(63 downto 0);
    mptl_config        : in  std_logic_vector(31 downto 0);
    -- Debug
    debug              : out std_logic_vector(255 downto 0));
  end component;

begin

  sim_mptl_g : if (TARGET_USE = SIM_MPTL) generate
    -- Define data during MPTL simulation
    dm_g : for n in 0 to DM_CHANNELS-1 generate
      direct_masters_m2s(n) <= dm_int_m2s(n);
      dm_int_s2m(n)         <= make_defined_s2m(direct_masters_s2m(n));
    end generate;
  end generate;

  -- Connect MPTL sideband inputs
  target_configured <= not(mptl_sb_t2b.mptl_target_configured_l);
  target_gtp_online <= not(mptl_sb_t2b.mptl_target_gtp_online_l);

  mptl128_interface_bridge_6t1_i : mptl128_interface_bridge_6t1
  port map(
    -- OCP Interface
    ocp_clk            => ocp_clk_in,        -- in    std_logic
    direct_slave_m2s   => direct_slave_m2s,  -- in    adb3_ocp_m2sT
    direct_slave_s2m   => direct_slave_s2m,  -- out   adb3_ocp_s2mT
    dma_channels_m2s   => dma_channels_m2s,  -- in    adb3_ocp_m2sT_array(0 to 3)
    dma_channels_s2m   => dma_channels_s2m,  -- out   adb3_ocp_s2mT_array(0 to 3)
    direct_masters_m2s => dm_int_m2s,        -- out   adb3_ocp_m2sT_array(0 to 2);
    direct_masters_s2m => dm_int_s2m,        -- in    adb3_ocp_s2mT_array(0 to 2);
    dma_abort          => dma_abort,         -- out   std_logic_vector(3 downto 0);
    -- MPTL Interface
    mptl_tx            => mptl_b2t,          -- out   mptl_pins_t;
    mptl_rx            => mptl_t2b,          -- in    mptl_pins_t;
    mptl_clk           => mptl_clk,          -- in    std_logic_dbl_t;
    -- MPTL Sideband Signals
    target_configured  => target_configured, -- in    std_logic
    target_gtp_online  => target_gtp_online, -- in    std_logic
    bridge_gtp_online  => bridge_gtp_online, -- out   std_logic
    -- MPTL Sideband Interface
    gpio_in            => gpio_b2t,          -- in    std_logic_vector(15 downto 0)
    gpio_out           => gpio_t2b,          -- out   std_logic_vector(15 downto 0)
    mptl_status        => mptl_status,       -- out   std_logic_vector(63 downto 0)
    mptl_config        => mptl_config,       -- in    std_logic_vector(31 downto 0)
    -- Debug
    debug              => open);             -- out   std_logic_vector(255 downto 0)

  -- Connect MPTL sideband ouputs
  mptl_sb_b2t.mptl_bridge_gtp_online_l <= not(bridge_gtp_online);

  -- Generate online signal
  mptl_online <= bridge_gtp_online and target_gtp_online and target_configured;

  -- Bridge ocp clock can be different from target ocp clock
  ocp_clk_out <= ocp_clk_in;

end;