--==============================================================================
--
--     File Name: mptl_if_target_netlist_wrap_6tge.vhd
--       Version: 0.1
--          Date: 18 Jul 2011
--         Model: Simulation only
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: Target MPTL interface netlist wrapper
--
--   Description: Wrapper for target MPTL interface simulation netlist
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2011 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 18 Jul 2011: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_inc_pkg.all;

entity mptl128_interface_target_6t1 is
port(
  -- OCP Interface
  ocp_clk                  : in    std_logic;
  direct_slave_m2s         : out   adb3_ocp_m2sT;
  direct_slave_s2m         : in    adb3_ocp_s2mT;
  dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to 3);
  dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to 3);
  dma_abort                : in    std_logic_vector(3 downto 0);
  -- MPTL Interface
  mptl_tx                  : out   mptl_pins_t;
  mptl_rx                  : in    mptl_pins_t;
  mptl_clk                 : in    std_logic_dbl_t;
  mptl_clk_out             : out   std_logic;
  -- Bridge to Target Sideband Signals
  mptl_target_configured_n : out   std_logic;
  mptl_target_gtp_online_n : out   std_logic;
  mptl_bridge_gtp_online_n : in    std_logic;
  -- MPTL64 Sideband Interface
  gpio_t2b                 : in    std_logic_vector(15 downto 0);
  gpio_b2t                 : out   std_logic_vector(15 downto 0));
end;

architecture sim_6tge of mptl128_interface_target_6t1 is

  signal direct_slave_m2s_i         : std_logic_vector(adb3_ocp_m2sT_slv_length-1 downto 0);
  signal direct_slave_s2m_i         : std_logic_vector(adb3_ocp_s2mT_slv_length-1 downto 0);
  signal dma_channels_m2s_i         : std_logic_vector(4*adb3_ocp_m2sT_slv_length-1 downto 0);
  signal dma_channels_s2m_i         : std_logic_vector(4*adb3_ocp_s2mT_slv_length-1 downto 0);

begin
  direct_slave_m2s <= to_adb3_ocp_m2sT(direct_slave_m2s_i);
  direct_slave_s2m_i <= to_std_logic_vector(direct_slave_s2m);

  dma_channels_m2s <= to_adb3_ocp_m2sT_array(dma_channels_m2s_i);
  dma_channels_s2m_i <= to_std_logic_vector(dma_channels_s2m);

  mptl128_interface_target_6t1_slv_i: entity work.mptl128_interface_target_6t1_slv
  port map (
    ocp_clk                  => ocp_clk,
    direct_slave_m2s         => direct_slave_m2s_i,
    direct_slave_s2m         => direct_slave_s2m_i,
    dma_channels_m2s         => dma_channels_m2s_i,
    dma_channels_s2m         => dma_channels_s2m_i,
    dma_abort                => dma_abort,
    mptl_tx_p                => mptl_tx.p(3 downto 0),
    mptl_tx_n                => mptl_tx.n(3 downto 0),
    mptl_rx_p                => mptl_rx.p(3 downto 0),
    mptl_rx_n                => mptl_rx.n(3 downto 0),
    mptl_clk_p               => mptl_clk.p,
    mptl_clk_n               => mptl_clk.n,
    mptl_clk_out             => mptl_clk_out,
    mptl_target_configured_n => mptl_target_configured_n,
    mptl_target_gtp_online_n => mptl_target_gtp_online_n,
    mptl_bridge_gtp_online_n => mptl_bridge_gtp_online_n,
    gpio_t2b                 => gpio_t2b,
    gpio_b2t                 => gpio_b2t);

end;