--==============================================================================
--
--     File Name: mptl_if_bridge_wrap_sim_6tl.vhd
--       Version: 0.1
--          Date: 18 Jul 2011
--         Model: Simulation only
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: Bridge MPTL interface wrapper
--
--   Description: Instantiate bridge MPTL interface OCP-only simulation code
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2011 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 18 Jul 2011:
--                     - ADM-XRC-6TL specific version of mptl_if_bridge_sim.
--                       moved from hdl/vhdl/common/mptl
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_inc_pkg.all;

entity mptl_if_bridge_wrap is
port(
  -- OCP Interface
  ocp_clk_in               : in    std_logic;
  direct_slave_m2s         : in    adb3_ocp_m2sT;
  direct_slave_s2m         : out   adb3_ocp_s2mT;
  dma_channels_m2s         : in    adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
  dma_channels_s2m         : out   adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
  direct_masters_m2s       : out   adb3_ocp_m2sT_array(0 to DM_CHANNELS-1);
  direct_masters_s2m       : in    adb3_ocp_s2mT_array(0 to DM_CHANNELS-1);
  dma_abort                : out   std_logic_vector(DMA_CHANNELS-1 downto 0);
  ocp_clk_out              : out   std_logic;
  -- MPTL Interface
  mptl_t2b                 : in    mptl_t2b_t;
  mptl_b2t                 : out   mptl_b2t_t;
  mptl_clk                 : in    std_logic_dbl_t;
  -- MPTL Initialisation
  mptl_sb_t2b              : in    mptl_sb_t2b_t;
  mptl_sb_b2t              : out   mptl_sb_b2t_t;
  mptl_online              : out   std_logic;
  -- MPTL Sideband
  gpio_b2t                 : in    std_logic_vector(15 downto 0);
  gpio_t2b                 : out   std_logic_vector(15 downto 0);
  mptl_status              : out   std_logic_vector(63 downto 0);
  mptl_config              : in    std_logic_vector(31 downto 0));
end;


architecture sim_6tl of mptl_if_bridge_wrap is

  signal online_count : std_logic_vector(5 downto 0) := "111111";
  signal online_cnt_0 : boolean := false;

  signal target_configured_l : std_logic;
  signal target_gtp_online_l : std_logic;
  signal bridge_gtp_online_l : std_logic;

  signal target_ocp_clk : std_logic;

begin

  -- Connect MPTL initialisation signals
  mptl_sb_b2t.mptl_bridge_gtp_online_l <= bridge_gtp_online_l;
  target_configured_l <= mptl_sb_t2b.mptl_target_configured_l;
  target_gtp_online_l <= mptl_sb_t2b.mptl_target_gtp_online_l;

  -- Simulation interface inputs
  target_ocp_clk     <= mptl_t2b.target_ocp_clk;
  direct_slave_s2m   <= mptl_t2b.direct_slave_s2m;
  dma_channels_s2m   <= mptl_t2b.dma_channels_s2m;
  direct_masters_m2s <= mptl_t2b.direct_masters_m2s;
  gpio_t2b           <= mptl_t2b.gpio_t2b;

  -- Simulation interface outputs
  mptl_b2t.direct_slave_m2s   <= direct_slave_m2s;
  mptl_b2t.dma_channels_m2s   <= dma_channels_m2s;
  mptl_b2t.direct_masters_s2m <= direct_masters_s2m;
  mptl_b2t.gpio_b2t           <= gpio_b2t;

  -- Online outputs
  online_i : process(target_ocp_clk)
  begin
    if rising_edge(target_ocp_clk) then
      if online_count = "000000" then
        online_cnt_0 <= true;
      else
        online_count <= online_count-1;
        online_cnt_0 <= false;
      end if;

      if (online_cnt_0) then
        bridge_gtp_online_l <= '0';
      else
        bridge_gtp_online_l <= '1';
      end if;
    end if;
  end process;

  -- Status outputs
  mptl_status <= (others => '0');

  -- Generate online signal
  mptl_online <= not(bridge_gtp_online_l or target_gtp_online_l or target_configured_l);

  -- dma_abort output directly on pins
  dma_abort <= not(mptl_sb_t2b.mptl_dma_abort_l);

  -- Direct OCP-OCP simulation
  -- Bridge ocp clock must be target ocp clock
  ocp_clk_out <= target_ocp_clk;

end;