#
# Script for Chipscope core generation
#
# Usage: xtclsh gen_chipscope.tcl [boardname]
# eg.
#        xtclsh gen_chipscope.tcl admxrc6t1
#
set _Board [lindex $argv 0]
#
set _ILA_Path  ${_Board}/${_Board}_ila.xco
set _ICON_Path ${_Board}/${_Board}_icon.xco
set _CGP_Path  ${_Board}/${_Board}.cgp
set _CGP_Dest  ${_Board}.cgp
#
#
if {[file exists ${_ILA_Path}]&&
    [file exists ${_ICON_Path}]&&
    [file exists ${_CGP_Path}]} then {
  puts ""
  puts "Board ${_Board} selected."
} else {
  puts ""
  puts "Missing files for board '${_Board}':"
  puts ""
  puts "CGP file: ${_CGP_Path}"
  puts "XCO file: ${_ILA_Path}"
  puts "XCO file: ${_ICON_Path}"
  puts ""
  puts "Please enter a valid board type, for example admxrc6t1."
  puts ""
  exit -1
}
#
#
puts ""
puts "Initialising project files..."
puts ""
puts "Deleting ngc dir ./${_Board}/ngc"
file delete -force ./${_Board}/ngc
puts "Making ngc dir ./${_Board}/ngc"
file mkdir ./${_Board}/ngc
puts "Copying CGP file ${_CGP_Path} to ./${_Board}/ngc/${_CGP_Dest}"
file copy ${_CGP_Path} ./${_Board}/ngc/${_CGP_Dest}
#
#
cd ./${_Board}/ngc
puts ""
puts "Generating chipscope_ila files:"
puts ""
if { [catch { exec coregen -r -b ../../${_ILA_Path} -p ${_CGP_Dest} >@ stdout } msg] } then {
  if { $errorCode ne "NONE" } then {
    puts ""
    puts "CORE Generator error detected:"
    puts ""
    puts "  errorCode: '$errorCode'"
    puts "  errorInfo: '$errorInfo'"
    cd ../..
    exit -1
  }
}

file delete -force coregen_ila.log
file rename -force coregen.log coregen_ila.log

puts ""
puts "Generating chipscope_icon files:"
puts ""
if { [catch { exec coregen -r -b ../../${_ICON_Path} -p ${_CGP_Dest} >@ stdout } msg] } then {
  if { $errorCode ne "NONE" } then {
    puts ""
    puts "CORE Generator error detected:"
    puts ""
    puts "  errorCode: '$errorCode'"
    puts "  errorInfo: '$errorInfo'"
    cd ../..
    exit -1
  }
}

file delete -force coregen_icon.log
file rename -force coregen.log coregen_icon.log
cd ../..

puts ""
puts "Generation of ChipScope cores complete."
puts ""
exit 0