#
# Script for checking if MIG files exist
#
# Usage: xtclsh check_mem_if.tcl [boardname]
# eg.
#        xtclsh check_mem_if.tcl admxrc6t1
#
set _Board [lindex $argv 0]
#
set _PRJ_Path ./${_Board}/${_Board}.prj
#
#
if {[file exists ${_PRJ_Path}]} then {
  puts ""
  puts "Board ${_Board} selected."
} else {
  puts ""
  puts "Missing files for board '${_Board}':"
  puts ""
  puts "PRJ file: ${_PRJ_Path}"
  puts ""
  puts "Please enter a valid board type, for example admxrc6t1."
  puts ""
  exit -1
}
#
#
# Get version number from project file
set prj_file [open "${_PRJ_Path}" r]
set file_data [read $prj_file]
close $prj_file

puts ""
puts "Searching ${_PRJ_Path} for MIG version..."
puts ""
set found 0
set data [split $file_data "\n"]
foreach line $data {
  if {[regexp -nocase "<version>(.*)</version>" ${line} result ver_num]} then {
    incr found
#    puts "  ($found): ${result}, ${ver_num}"
  }
}

if {${found} eq 1} then {
  set _MIG_Version v[regsub {\.} ${ver_num} "_"]
  puts "Selecting MIG version ${ver_num} (${_MIG_Version})"
} else {
  puts "Error: Found ${found} version entries in ${_PRJ_Path}"
  exit -1
}
#
#
# Get controllers number from project file
puts ""
puts "Searching ${_PRJ_Path} for MIG controllers..."
puts ""
set found 0
set data [split $file_data "\n"]
foreach line $data {
  if {[regexp -nocase "NoOfControllers=\"(.*)\"" ${line} result con_num]} then {
    incr found
#    puts "  ($found): ${result}, ${con_num}"
  }
}

if {${found} eq 1} then {
  set _MIG_Con ${con_num}
  puts "Selecting MIG controllers = ${_MIG_Con}"
} else {
  puts "Error: Found ${found} controller entries in ${_PRJ_Path}"
  exit -1
}


set _MIG_Path ${_Board}/rtl/mig_${_MIG_Version}
set _Error 0
#
#
proc test_file {_File} {
  global _Error
  if {![file exists $_File]} then {
    puts "$_File missing"
    incr _Error
  } else {
    puts "$_File found"
  }
}
#
#
puts ""
puts "Checking MIG ${_MIG_Version} files (${_MIG_Con} controllers):"
#
test_file ${_MIG_Path}/ui/ui_cmd.vhd
test_file ${_MIG_Path}/ui/ui_rd_data.vhd
test_file ${_MIG_Path}/ui/ui_wr_data.vhd
test_file ${_MIG_Path}/ui/ui_top.vhd
#
test_file ${_MIG_Path}/phy/circ_buffer.vhd
test_file ${_MIG_Path}/phy/phy_ck_iob.vhd
test_file ${_MIG_Path}/phy/phy_clock_io.vhd
test_file ${_MIG_Path}/phy/phy_control_io.vhd
test_file ${_MIG_Path}/phy/phy_data_io.vhd
test_file ${_MIG_Path}/phy/phy_dly_ctrl.vhd
test_file ${_MIG_Path}/phy/phy_dm_iob.vhd
test_file ${_MIG_Path}/phy/phy_dq_iob.vhd
test_file ${_MIG_Path}/phy/phy_dqs_iob.vhd
test_file ${_MIG_Path}/phy/phy_init.vhd
test_file ${_MIG_Path}/phy/phy_pd.vhd
test_file ${_MIG_Path}/phy/phy_pd_top.vhd
test_file ${_MIG_Path}/phy/phy_rdclk_gen.vhd
test_file ${_MIG_Path}/phy/phy_rdctrl_sync.vhd
test_file ${_MIG_Path}/phy/phy_rddata_sync.vhd
test_file ${_MIG_Path}/phy/phy_rdlvl.vhd
test_file ${_MIG_Path}/phy/phy_read.vhd
test_file ${_MIG_Path}/phy/phy_write.vhd
test_file ${_MIG_Path}/phy/phy_wrlvl.vhd
test_file ${_MIG_Path}/phy/rd_bitslip.vhd
test_file ${_MIG_Path}/phy/phy_top.vhd
#
test_file ${_MIG_Path}/controller/arb_mux.vhd
test_file ${_MIG_Path}/controller/arb_row_col.vhd
test_file ${_MIG_Path}/controller/arb_select.vhd
test_file ${_MIG_Path}/controller/bank_cntrl.vhd
test_file ${_MIG_Path}/controller/bank_common.vhd
test_file ${_MIG_Path}/controller/bank_compare.vhd
test_file ${_MIG_Path}/controller/bank_mach.vhd
test_file ${_MIG_Path}/controller/bank_queue.vhd
test_file ${_MIG_Path}/controller/bank_state.vhd
test_file ${_MIG_Path}/controller/col_mach.vhd
test_file ${_MIG_Path}/controller/rank_cntrl.vhd
test_file ${_MIG_Path}/controller/rank_common.vhd
test_file ${_MIG_Path}/controller/rank_mach.vhd
test_file ${_MIG_Path}/controller/round_robin_arb.vhd
test_file ${_MIG_Path}/controller/mc.vhd
#
test_file ${_MIG_Path}/ip_top/mem_intfc.vhd
test_file ${_MIG_Path}/ip_top/infrastructure.vhd
#
if {${_MIG_Con} >= 1} then { test_file ${_MIG_Path}/ip_top/c0_memc_ui_top.vhd }
if {${_MIG_Con} >= 2} then { test_file ${_MIG_Path}/ip_top/c1_memc_ui_top.vhd }
if {${_MIG_Con} >= 3} then { test_file ${_MIG_Path}/ip_top/c2_memc_ui_top.vhd }
if {${_MIG_Con} >= 4} then { test_file ${_MIG_Path}/ip_top/c3_memc_ui_top.vhd }
#
#
puts ""
puts "CORE checker complete with ${_Error} errors detected."
puts ""
#
if {${_Error} eq 0} then {
  exit 0
} else {
  puts ""
  puts "All DDR3 MIG ${_MIG_Version} files for ${_Board} are not found."
  puts ""
  puts "Please re-run this macro file after generating the"
  puts "DDR3 MIG ${_MIG_Version} files by running"
  puts ""
  puts "  xtclsh gen_mem_if.tcl ${_Board}"
  puts ""
  puts "in a command console in directory:"
  puts ""
  puts "  hdl/vhdl/common/mem_if/ddr3_sdram"
  puts ""
  exit -1
}