#
# Script for MIG using ISE12.3 onwards
#
# Usage: xtclsh gen_mem_if.tcl [boardname]
# eg.
#        xtclsh gen_mem_if.tcl admxrc6t1
#
set _Board [lindex $argv 0]
#
set _XCO_Path ./${_Board}/${_Board}.xco
set _CGP_Path ./${_Board}/${_Board}.cgp
set _PRJ_Path ./${_Board}/${_Board}.prj
set _CUS_Path ./custom_part
#
#
if {[file exists ${_XCO_Path}]&&
    [file exists ${_CGP_Path}]&&
    [file exists ${_CUS_Path}]&&
    [file exists ${_PRJ_Path}]} then {
  puts ""
  puts "Board ${_Board} selected."
} else {
  puts ""
  puts "Missing files for board '${_Board}':"
  puts ""
  puts "CGP file: ${_CGP_Path}"
  puts "XCO file: ${_XCO_Path}"
  puts "PRJ file: ${_PRJ_Path}"
  puts "Part dir: ${_CUS_Path}"
  puts ""
  puts "Please enter a valid board type, for example admxrc6t1."
  puts ""
  exit -1
}
#
#
# Get version number from project file
set prj_file [open "${_PRJ_Path}" r]
set file_data [read $prj_file]
close $prj_file

puts ""
puts "Searching ${_PRJ_Path} for MIG version..."
puts ""
set found 0
set data [split $file_data "\n"]
foreach line $data {
  if {[regexp -nocase "<version>(.*)</version>" ${line} result ver_num]} then {
    incr found
    puts "  ($found): ${result}, ${ver_num}"
  }
}

if {${found} eq 1} then {
  set _MIG_Version mig_v[regsub {\.} ${ver_num} "_"]
  puts "Selecting MIG version ${ver_num} (${_MIG_Version})"
} else {
  puts "Error: Found ${found} version entries in ${_PRJ_Path}"
  exit -1
}
#
#
set _MIG_Temp ./${_Board}/mig_temp
#
set _CGP_Dest ${_Board}.cgp
set _PRJ_Dest ${_MIG_Version}/user_design/mig.prj
set _XCO_Dest ${_Board}.xco
set _CUS_Dest custom_part
#
#
puts ""
puts "Initialising project files..."
puts ""
puts "Deleting temp dir ${_MIG_Temp}"
file delete -force ${_MIG_Temp}
puts "Making temp dir ${_MIG_Temp}/${_MIG_Version}/user_design"
file mkdir ${_MIG_Temp}/${_MIG_Version}/user_design
puts "Copying CGP file ${_CGP_Path} to ${_MIG_Temp}/${_CGP_Dest}"
file copy ${_CGP_Path} ${_MIG_Temp}/${_CGP_Dest}
puts "Copying PRJ file ${_PRJ_Path} to ${_MIG_Temp}/${_PRJ_Dest}"
file copy ${_PRJ_Path} ${_MIG_Temp}/${_PRJ_Dest}
puts "Copying XCO file ${_XCO_Path} to ${_MIG_Temp}/${_XCO_Dest}"
file copy ${_XCO_Path} ${_MIG_Temp}/${_XCO_Dest}
puts "Copying custom parts ${_CUS_Path} to ${_MIG_Temp}/${_CUS_Dest}"
file copy ${_CUS_Path} ${_MIG_Temp}/${_CUS_Dest}
#
#
puts ""
puts "Generating MIG files:"
puts ""
puts "Executing coregen -r -b ${_XCO_Dest}  -p ${_CGP_Dest}"
#
cd ${_MIG_Temp}
if { [catch { exec coregen -r -b ${_XCO_Dest} -p ${_CGP_Dest} >@ stdout } msg] } then {
  if { $errorCode ne "NONE" } then {
    puts ""
    puts "CORE Generator error detected:"
    puts ""
    puts "  errorCode: '$errorCode'"
    puts "  errorInfo: '$errorInfo'"
    cd ../..
    exit -1
  }
}
cd ../..
#
#
set _RTL_Path ./${_Board}/rtl/${_MIG_Version}
puts ""
puts "Copying generated VHDL to RTL dir ${_RTL_Path}"
puts ""
puts "Deleting old rtl dir ${_RTL_Path}"
file delete -force ${_RTL_Path}
puts "Making rtl dir ${_RTL_Path}"
file mkdir ${_RTL_Path}
puts "Copying rtl files ${_MIG_Temp}/${_MIG_Version}/user_design/rtl to ${_RTL_Path}"
foreach _RTL_Dir [glob -directory ${_MIG_Temp}/${_MIG_Version}/user_design/rtl *] {
  file copy ${_RTL_Dir} ${_RTL_Path}
}
puts "Copying log file ${_MIG_Temp}/coregen.log to ${_RTL_Path}"
file copy ${_MIG_Temp}/coregen.log ${_RTL_Path}
#
puts ""
puts "Generation of MIG core ${_MIG_Version} for ${_Board} is complete."
puts ""
exit 0