--==============================================================================
--
--     File Name: ddr3_sdram_dly_cnt.vhd
--       Version: 0.1
--          Date: 01 Apr 2009
--         Model: Behavioral (NOT SYNTHESISABLE)
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: Generic DDR3 SDRAM model
--
--   Description: Delay counter for cycle timing verification
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2009 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 01 Apr 09: Initial design
--
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;


entity ddr3_sdram_dly_cnt is
  port(
    ck       : in    std_logic;
    delay_go : in    boolean;    -- Initiate delay count
    delay    : in    natural;    -- Initial delay count value
    delay_tc : out   boolean);   -- End of delay count
  end ddr3_sdram_dly_cnt;


architecture behavioural of ddr3_sdram_dly_cnt is

  signal delay_cnt : natural;

begin

  -- Delay counter
  dly_cnt : process(ck)
  begin
    if (ck'event and ck = '1') then
      if (delay_go) then
        if ((delay - 1) < 1) then
          delay_tc  <= true;
          delay_cnt <= 0;
        else
          delay_tc  <= false;
          delay_cnt <= delay - 1;
        end if;
      else
        if (delay_cnt > 0) then
          delay_cnt <= delay_cnt - 1;
        end if;
        if (delay_cnt < 2) then
          delay_tc <= true;   -- Initialise to true. Delay count = 1 is used by the next command
        else
          delay_tc <= false;
        end if;
      end if;
    end if;
  end process;

end behavioural;
