--==============================================================================
--
--     File Name: test_uber_mem_6t1.vhd
--       Version: 0.1
--          Date: 29 Sep 2011
--         Model: Simulation only
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: ADB3 Target Design Test: Model on-board memory
--
--   Description: Instantiates models for on-board memories
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2011 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 29 Sep 2011: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_target_inc_pkg.all;
use work.adb3_target_tb_inc_pkg.all;
use work.uber_tb_pkg.all;
use work.ddr3_sdram_pkg.all;

entity test_uber_mem is
generic(
  message_level : natural);
port(
  -- OCP Interface
  mem_addr_out   : in    mem_addr_out_t;
  mem_ctrl_out   : in    mem_ctrl_out_t;
  mem_data_inout : inout mem_data_inout_t;
  mem_clk_out    : in    mem_clk_out_t);
end test_uber_mem;

architecture sim_6t1 of test_uber_mem is

begin

  ----------------------------
  -- DDR3 SDRAM Bank Models --
  ----------------------------

  ddr3_model_g : for n in 0 to DDR3_BANKS-1 generate
    -- Instantiate LS data RAM model
    ddr3_sdram_bank_ls_i : ddr3_sdram
      generic map(
        message_level  => message_level,   -- natural;
        part           => BOARD_DDR3_PART, -- part_t;
        short_init_dly => true)            -- boolean);
      port map(
        ck             => mem_clk_out.ddr3_clk_out(n).clk_p(0),                                  -- in    std_logic;
        ck_l           => mem_clk_out.ddr3_clk_out(n).clk_n(0),                                  -- in    std_logic;
        reset_l        => mem_ctrl_out.ddr3_ctrl_out(n).reset_l,                                 -- in    std_logic;
        cke            => mem_ctrl_out.ddr3_ctrl_out(n).cke(0),                                  -- in    std_logic;
        cs_l           => mem_ctrl_out.ddr3_ctrl_out(n).cs_l(0),                                 -- in    std_logic;
        ras_l          => mem_ctrl_out.ddr3_ctrl_out(n).ras_l,                                   -- in    std_logic;
        cas_l          => mem_ctrl_out.ddr3_ctrl_out(n).cas_l,                                   -- in    std_logic;
        we_l           => mem_ctrl_out.ddr3_ctrl_out(n).we_l,                                    -- in    std_logic;
        odt            => mem_ctrl_out.ddr3_ctrl_out(n).odt(0),                                  -- in    std_logic;
        dm             => mem_ctrl_out.ddr3_ctrl_out(n).dm(1 downto 0),                          -- in    std_logic_vector(part.part_size.data_bytes-1 downto 0);
        ba             => mem_ctrl_out.ddr3_ctrl_out(n).ba,                                      -- in    std_logic_vector(part.part_size.bank_width-1 downto 0);
        a              => mem_addr_out.ddr3_addr_out(n).a(BOARD_DDR3_BANK_ROW_WIDTH-1 downto 0), -- in    std_logic_vector(part.part_size.arow_width-1 downto 0);
        dq             => mem_data_inout.ddr3_data_inout(n).dq(15 downto 0),                     -- inout std_logic_vector(part.part_size.data_width-1 downto 0);
        dqs            => mem_data_inout.ddr3_data_inout(n).dqs_p(1 downto 0),                   -- inout std_logic_vector(part.part_size.data_bytes-1 downto 0);
        dqs_l          => mem_data_inout.ddr3_data_inout(n).dqs_n(1 downto 0));                  -- inout std_logic_vector(part.part_size.data_bytes-1 downto 0);

    -- Instantiate MS data RAM model
    ddr3_sdram_bank_ms_i : ddr3_sdram
      generic map(
        message_level  => message_level,   -- natural;
        part           => BOARD_DDR3_PART, -- part_t;
        short_init_dly => true)            -- boolean);
      port map(
        ck             => mem_clk_out.ddr3_clk_out(n).clk_p(0),                                  -- in    std_logic;
        ck_l           => mem_clk_out.ddr3_clk_out(n).clk_n(0),                                  -- in    std_logic;
        reset_l        => mem_ctrl_out.ddr3_ctrl_out(n).reset_l,                                 -- in    std_logic;
        cke            => mem_ctrl_out.ddr3_ctrl_out(n).cke(0),                                  -- in    std_logic;
        cs_l           => mem_ctrl_out.ddr3_ctrl_out(n).cs_l(0),                                 -- in    std_logic;
        ras_l          => mem_ctrl_out.ddr3_ctrl_out(n).ras_l,                                   -- in    std_logic;
        cas_l          => mem_ctrl_out.ddr3_ctrl_out(n).cas_l,                                   -- in    std_logic;
        we_l           => mem_ctrl_out.ddr3_ctrl_out(n).we_l,                                    -- in    std_logic;
        odt            => mem_ctrl_out.ddr3_ctrl_out(n).odt(0),                                  -- in    std_logic;
        dm             => mem_ctrl_out.ddr3_ctrl_out(n).dm(3 downto 2),                          -- in    std_logic_vector(part.part_size.data_bytes-1 downto 0);
        ba             => mem_ctrl_out.ddr3_ctrl_out(n).ba,                                      -- in    std_logic_vector(part.part_size.bank_width-1 downto 0);
        a              => mem_addr_out.ddr3_addr_out(n).a(BOARD_DDR3_BANK_ROW_WIDTH-1 downto 0), -- in    std_logic_vector(part.part_size.arow_width-1 downto 0);
        dq             => mem_data_inout.ddr3_data_inout(n).dq(31 downto 16),                    -- inout std_logic_vector(part.part_size.data_width-1 downto 0);
        dqs            => mem_data_inout.ddr3_data_inout(n).dqs_p(3 downto 2),                   -- inout std_logic_vector(part.part_size.data_bytes-1 downto 0);
        dqs_l          => mem_data_inout.ddr3_data_inout(n).dqs_n(3 downto 2));                  -- inout std_logic_vector(part.part_size.data_bytes-1 downto 0);

    ddr3_check_g : if (DDR3_BANK_ROW_WIDTH > BOARD_DDR3_BANK_ROW_WIDTH) generate
      -- Check for activity on unused address bits
      assert not(OR_reduce(mem_addr_out.ddr3_addr_out(n).a(DDR3_BANK_ROW_WIDTH-1 downto BOARD_DDR3_BANK_ROW_WIDTH)) = '1')
        report "(test_uber): ERROR: Activity on unused DDR3 SDRAM address bits a(" & integer'image(DDR3_BANK_ROW_WIDTH-1) &
               ":" & integer'image(BOARD_DDR3_BANK_ROW_WIDTH) & ")"
        severity error;
    end generate;
  end generate;

end;