--==============================================================================
--
--     File Name: blk_ds_simple_test.vhd
--       Version: 0.3
--          Date: 16 Feb 2011
--         Model: Synthesisable
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: Simple test
--
--   Description: Simple test
--                Connected to Direct Slave OCP interface
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2010-2011 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 21 Apr 2010: Initial design
--                0.2, TW,  09 Feb 2011:
--                  - Now references adb3_ocp_comp package instead of
--                    adb3_target_pkg.
--                  - Removed some unnecessary resets.
--                0.3, KDC, 16 Feb 2011:
--                  - Changed range of addr_width to match OCP profile
--
--==============================================================================


library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_ocp_comp.all;
use work.uber_pkg.all;

entity blk_ds_simple_test is
generic(
  addr_width : natural range 1 to ADB3_OCP_ADDR_WIDTH);
port(
  -- OCP interface
  ocp_rst          : in    std_logic;
  ocp_clk          : in    std_logic;
  direct_slave_m2s : in    adb3_ocp_m2sT;
  direct_slave_s2m : out   adb3_ocp_s2mT);
end;


architecture syn of blk_ds_simple_test is

  constant REG_WIDTH : natural := 32;
  constant WE_WIDTH  : natural := REG_WIDTH/8;

  -- Register interface signals
  signal ds_d  : std_logic_vector(REG_WIDTH-1 downto 0);
  signal ds_q  : std_logic_vector(REG_WIDTH-1 downto 0);
  signal ds_a  : std_logic_vector(addr_width-1 downto 0);
  signal ds_w  : std_logic;
  signal ds_r  : std_logic;
  signal ds_we : std_logic_vector(WE_WIDTH-1 downto 0);

  -- Simple test
  signal wdata_reg : std_logic_vector(REG_WIDTH-1 downto 0);
  signal rdata_reg : std_logic_vector(REG_WIDTH-1 downto 0);

begin

  --------------------------------
  -- Direct Slave OCP interface --
  --------------------------------

  adb3_ocp_simple_bus_if_i : adb3_ocp_simple_bus_if
  generic map(
    addr_width   => addr_width, -- natural range 1 to ADB3_OCP_ADDR_WIDTH
    data_width   => REG_WIDTH,  -- natural range 8 to ADB3_OCP_DATA_WIDTH
    read_latency => 1)          -- natural := 1
  port map(
    -- OCP Interface
    ocp_rst => ocp_rst,           -- in    std_logic
    ocp_clk => ocp_clk,           -- in    std_logic
    ocp_m2s => direct_slave_m2s,  -- in    adb3_ocp_m2sT
    ocp_s2m => direct_slave_s2m,  -- out   adb3_ocp_s2mT
    -- Simple bus interface
    d       => ds_d,        -- out   std_logic_vector(data_width-1 downto 0)
    q       => ds_q,        -- in    std_logic_vector(data_width-1 downto 0)
    a       => ds_a,        -- out   std_logic_vector(addr_width-1 downto 0)
    w       => ds_w,        -- out   std_logic
    r       => ds_r,        -- out   std_logic
    we      => ds_we);      -- out   std_logic_vector((data_width/8)-1 downto 0))


  -- Update write registers
  wr_regs_p : process(ocp_rst,ocp_clk)
    variable u,l : natural;
  begin
    if (ocp_rst = '1') then
      wdata_reg <= (others => '0');
    elsif rising_edge(ocp_clk) then
      for i in 0 to WE_WIDTH-1 loop
        u := 8*(i+1)-1;
        l := 8*i;
        if (ds_we(i)='1') then
          if (ds_a = SIMPLE_WDATA_REG_ADDR(addr_width-1 downto 0)) then
            wdata_reg(u downto l) <= ds_d(u downto l);
          end if;
        end if;
      end loop;
    end if;
  end process;


  -- Multiplex read registers
  rd_regs_p : process(ocp_clk)
    variable ds_q_var : std_logic_vector(REG_WIDTH-1 downto 0);
  begin
    if rising_edge(ocp_clk) then
      ds_q_var := (others => '-');

      if (ds_a = SIMPLE_RDATA_REG_ADDR(addr_width-1 downto 0)) then
        ds_q_var := rdata_reg;
      end if;

      ds_q <= ds_q_var;
    end if;
  end process;


  rdata_reg <= wdata_reg(3  downto 0)  & wdata_reg(7  downto 4)  &
               wdata_reg(11 downto 8)  & wdata_reg(15 downto 12) &
               wdata_reg(19 downto 16) & wdata_reg(23 downto 20) &
               wdata_reg(27 downto 24) & wdata_reg(31 downto 28);

end;